@@ -25,7 +25,6 @@
 #include <string.h>
 #include <ctype.h>
 #include <assert.h>
-#include <unistd.h>
 
 #include "agent.h"
 #include "../common/i18n.h"
@@ -34,7 +33,7 @@
 
 static int
 store_key (gcry_sexp_t private, const char *passphrase, int force,
-           unsigned long s2k_count, time_t timestamp)
+	unsigned long s2k_count)
 {
   int rc;
   unsigned char *buf;
@@ -69,8 +68,7 @@ store_key (gcry_sexp_t private, const char *passphrase, int force,
       buf = p;
     }
 
-  rc = agent_write_private_key (grip, buf, len, force, timestamp,
-                                NULL, NULL, NULL);
+  rc = agent_write_private_key (grip, buf, len, force);
   xfree (buf);
   return rc;
 }
@@ -92,82 +90,60 @@ nonalpha_count (const char *s)
 
 
 /* Check PW against a list of pattern.  Return 0 if PW does not match
-   these pattern.  If CHECK_CONSTRAINTS_NEW_SYMKEY is set in flags and
-   --check-sym-passphrase-pattern has been configured, use the pattern
-   file from that option.  */
+   these pattern.  */
 static int
-do_check_passphrase_pattern (ctrl_t ctrl, const char *pw, unsigned int flags)
+check_passphrase_pattern (ctrl_t ctrl, const char *pw)
 {
   gpg_error_t err = 0;
   const char *pgmname = gnupg_module_name (GNUPG_MODULE_NAME_CHECK_PATTERN);
-  estream_t stream_to_check_pattern = NULL;
+  FILE *infp;
   const char *argv[10];
   pid_t pid;
   int result, i;
-  const char *pattern;
-  char *patternfname;
 
   (void)ctrl;
 
-  pattern = opt.check_passphrase_pattern;
-  if ((flags & CHECK_CONSTRAINTS_NEW_SYMKEY)
-      && opt.check_sym_passphrase_pattern)
-    pattern = opt.check_sym_passphrase_pattern;
-  if (!pattern)
-    return 1; /* Oops - Assume password should not be used  */
-
-  if (strchr (pattern, '/') || strchr (pattern, '\\')
-      || (*pattern == '~' && pattern[1] == '/'))
-    patternfname = make_absfilename_try (pattern, NULL);
-  else
-    patternfname = make_filename_try (gnupg_sysconfdir (), pattern, NULL);
-  if (!patternfname)
+  infp = gnupg_tmpfile ();
+  if (!infp)
     {
-      log_error ("error making filename from '%s': %s\n",
-                 pattern, gpg_strerror (gpg_error_from_syserror ()));
-      return 1; /* Do not pass the check.  */
+      err = gpg_error_from_syserror ();
+      log_error (_("error creating temporary file: %s\n"), gpg_strerror (err));
+      return 1; /* Error - assume password should not be used.  */
     }
 
-  /* Make debugging a broken config easier by printing a useful error
-   * message.  */
-  if (gnupg_access (patternfname, F_OK))
+  if (fwrite (pw, strlen (pw), 1, infp) != 1)
     {
-      log_error ("error accessing '%s': %s\n",
-                 patternfname, gpg_strerror (gpg_error_from_syserror ()));
-      xfree (patternfname);
-      return 1; /* Do not pass the check.  */
+      err = gpg_error_from_syserror ();
+      log_error (_("error writing to temporary file: %s\n"),
+                 gpg_strerror (err));
+      fclose (infp);
+      return 1; /* Error - assume password should not be used.  */
     }
+  fseek (infp, 0, SEEK_SET);
+  clearerr (infp);
 
   i = 0;
   argv[i++] = "--null";
   argv[i++] = "--",
-  argv[i++] = patternfname,
+  argv[i++] = opt.check_passphrase_pattern,
   argv[i] = NULL;
   assert (i < sizeof argv);
 
-  if (gnupg_spawn_process (pgmname, argv, NULL, NULL, 0,
-                           &stream_to_check_pattern, NULL, NULL, &pid))
+  if (gnupg_spawn_process_fd (pgmname, argv, fileno (infp), -1, -1, &pid))
     result = 1; /* Execute error - assume password should no be used.  */
+  else if (gnupg_wait_process (pgmname, pid, 1, NULL))
+    result = 1; /* Helper returned an error - probably a match.  */
   else
-    {
-      es_set_binary (stream_to_check_pattern);
-      if (es_fwrite (pw, strlen (pw), 1, stream_to_check_pattern) != 1)
-        {
-          err = gpg_error_from_syserror ();
-          log_error (_("error writing to pipe: %s\n"), gpg_strerror (err));
-          result = 1; /* Error - assume password should not be used.  */
-        }
-      else
-        es_fflush (stream_to_check_pattern);
-      es_fclose (stream_to_check_pattern);
-      if (gnupg_wait_process (pgmname, pid, 1, NULL))
-        result = 1; /* Helper returned an error - probably a match.  */
-      else
-        result = 0; /* Success; i.e. no match.  */
-      gnupg_release_process (pid);
-    }
+    result = 0; /* Success; i.e. no match.  */
+  gnupg_release_process (pid);
 
-  xfree (patternfname);
+  /* Overwrite our temporary file. */
+  fseek (infp, 0, SEEK_SET);
+  clearerr (infp);
+  for (i=((strlen (pw)+99)/100)*100; i > 0; i--)
+    putc ('\xff', infp);
+  fflush (infp);
+  fclose (infp);
   return result;
 }
 
@@ -198,17 +174,12 @@ take_this_one_anyway (ctrl_t ctrl, const char *desc)
 
 
 /* Check whether the passphrase PW is suitable. Returns 0 if the
- * passphrase is suitable and true if it is not and the user should be
- * asked to provide a different one.  If FAILED_CONSTRAINT is set, a
- * message describing the problem is returned at FAILED_CONSTRAINT.
- * The FLAGS are:
- *   CHECK_CONSTRAINTS_NOT_EMPTY
- *       Do not allow an empty passphrase
- *   CHECK_CONSTRAINTS_NEW_SYMKEY
- *       Hint that the passphrase is used for a new symmetric key.
- */
+   passphrase is suitable and true if it is not and the user should be
+   asked to provide a different one.  If FAILED_CONSTRAINT is set, a
+   message describing the problem is returned in
+   *FAILED_CONSTRAINT.  */
 int
-check_passphrase_constraints (ctrl_t ctrl, const char *pw, unsigned int flags,
+check_passphrase_constraints (ctrl_t ctrl, const char *pw,
 			      char **failed_constraint)
 {
   gpg_error_t err = 0;
@@ -217,7 +188,6 @@ check_passphrase_constraints (ctrl_t ctrl, const char *pw, unsigned int flags,
   char *msg1 = NULL;
   char *msg2 = NULL;
   char *msg3 = NULL;
-  int no_empty = !!(flags & CHECK_CONSTRAINTS_NOT_EMPTY);
 
   if (ctrl && ctrl->pinentry_mode == PINENTRY_MODE_LOOPBACK)
     return 0;
@@ -228,7 +198,7 @@ check_passphrase_constraints (ctrl_t ctrl, const char *pw, unsigned int flags,
   /* The first check is to warn about an empty passphrase. */
   if (!*pw)
     {
-      const char *desc = (opt.enforce_passphrase_constraints || no_empty?
+      const char *desc = (opt.enforce_passphrase_constraints?
                           L_("You have not entered a passphrase!%0A"
                              "An empty passphrase is not allowed.") :
                           L_("You have not entered a passphrase - "
@@ -239,7 +209,7 @@ check_passphrase_constraints (ctrl_t ctrl, const char *pw, unsigned int flags,
       err = 1;
       if (failed_constraint)
 	{
-	  if (opt.enforce_passphrase_constraints || no_empty)
+	  if (opt.enforce_passphrase_constraints)
 	    *failed_constraint = xstrdup (desc);
 	  else
 	    err = take_this_one_anyway2 (ctrl, desc,
@@ -295,9 +265,8 @@ check_passphrase_constraints (ctrl_t ctrl, const char *pw, unsigned int flags,
      and pattern.  The actual test is done by an external program.
      The warning message is generic to give the user no hint on how to
      circumvent this list.  */
-  if (*pw
-      && (opt.check_passphrase_pattern || opt.check_sym_passphrase_pattern)
-      && do_check_passphrase_pattern (ctrl, pw, flags))
+  if (*pw && opt.check_passphrase_pattern &&
+      check_passphrase_pattern (ctrl, pw))
     {
       if (!failed_constraint)
         {
@@ -412,12 +381,12 @@ agent_ask_new_passphrase (ctrl_t ctrl, const char *prompt,
   if (!pi2)
     {
       err = gpg_error_from_syserror ();
-      xfree (pi);
+      xfree (pi2);
       return err;
     }
   pi->max_length = MAX_PASSPHRASE_LEN + 1;
   pi->max_tries = 3;
-  pi->with_qualitybar = 0;
+  pi->with_qualitybar = 1;
   pi->with_repeat = 1;
   pi2->max_length = MAX_PASSPHRASE_LEN + 1;
   pi2->max_tries = 3;
@@ -430,7 +399,7 @@ agent_ask_new_passphrase (ctrl_t ctrl, const char *prompt,
   initial_errtext = NULL;
   if (!err)
     {
-      if (check_passphrase_constraints (ctrl, pi->pin, 0, &initial_errtext))
+      if (check_passphrase_constraints (ctrl, pi->pin, &initial_errtext))
         {
           pi->failed_tries = 0;
           pi2->failed_tries = 0;
@@ -472,11 +441,9 @@ agent_ask_new_passphrase (ctrl_t ctrl, const char *prompt,
    KEYPARAM.  If CACHE_NONCE is given first try to lookup a passphrase
    using the cache nonce.  If NO_PROTECTION is true the key will not
    be protected by a passphrase.  If OVERRIDE_PASSPHRASE is true that
-   passphrase will be used for the new key.  If TIMESTAMP is not zero
-   it will be recorded as creation date of the key (unless extended
-   format is disabled) . */
+   passphrase will be used for the new key.  */
 int
-agent_genkey (ctrl_t ctrl, const char *cache_nonce, time_t timestamp,
+agent_genkey (ctrl_t ctrl, const char *cache_nonce,
               const char *keyparam, size_t keyparamlen, int no_protection,
               const char *override_passphrase, int preset, membuf_t *outbuf)
 {
@@ -550,7 +517,7 @@ agent_genkey (ctrl_t ctrl, const char *cache_nonce, time_t timestamp,
   /* store the secret key */
   if (DBG_CRYPTO)
     log_debug ("storing private key\n");
-  rc = store_key (s_private, passphrase, 0, ctrl->s2k_count, timestamp);
+  rc = store_key (s_private, passphrase, 0, ctrl->s2k_count);
   if (!rc)
     {
       if (!cache_nonce)
@@ -624,7 +591,7 @@ agent_protect_and_store (ctrl_t ctrl, gcry_sexp_t s_skey,
     {
       /* Take an empty string as request not to protect the key.  */
       err = store_key (s_skey, **passphrase_addr? *passphrase_addr:NULL, 1,
-                       ctrl->s2k_count, 0);
+	      ctrl->s2k_count);
     }
   else
     {
@@ -639,7 +606,7 @@ agent_protect_and_store (ctrl_t ctrl, gcry_sexp_t s_skey,
                                       L_("Please enter the new passphrase"),
                                       &pass);
       if (!err)
-        err = store_key (s_skey, pass, 1, ctrl->s2k_count, 0);
+        err = store_key (s_skey, pass, 1, ctrl->s2k_count);
       if (!err && passphrase_addr)
         *passphrase_addr = pass;
       else
