@@ -124,7 +124,7 @@
    that the handle shall only be used by one thread at a time.  This
    function creates a unique file temporary file (".#lk*") in the same
    directory as FNAME and returns a handle for further operations.
-   The module keeps track of these unique files so that they will be
+   The module keeps track of theses unique files so that they will be
    unlinked using the atexit handler.  If you don't need the lock file
    anymore, you may also explicitly remove it with a call to:
 
@@ -140,7 +140,7 @@
    you pass (0) instead of (-1) the function does not wait in case the
    file is already locked but returns -1 and sets ERRNO to EACCES.
    Any other positive value for the second parameter is considered a
-   timeout value in milliseconds.
+   timeout valuie in milliseconds.
 
    To release the lock you call:
 
@@ -209,6 +209,8 @@
      EXTSEP_S            - Separation string for file name suffixes.
                            Usually not redefined.
 
+     HAVE_W32CE_SYSTEM   - Currently only used by GnuPG.
+
    Note that there is a test program t-dotlock which has compile
    instructions at its end.  At least for SMBFS and CIFS it is
    important that 64 bit versions of stat are used; most programming
@@ -291,7 +293,6 @@
 # include <sys/types.h>
 # include <sys/stat.h>
 # include <sys/utsname.h>
-# include <dirent.h>
 #endif
 #include <sys/types.h>
 #include <sys/time.h>
@@ -313,6 +314,9 @@
 # include "common-defs.h"
 # include "stringhelp.h"  /* For stpcpy and w32_strerror. */
 #endif
+#ifdef HAVE_W32CE_SYSTEM
+# include "utf8conv.h"  /* WindowsCE requires filename conversion.  */
+#endif
 
 #include "dotlock.h"
 
@@ -394,18 +398,9 @@ struct dotlock_handle
   unsigned int locked:1;     /* Lock status.                          */
   unsigned int disable:1;    /* If true, locking is disabled.         */
   unsigned int use_o_excl:1; /* Use open (O_EXCL) for locking.        */
-  unsigned int by_parent:1;  /* Parent does the locking.              */
-  unsigned int no_write:1;   /* No write to the lockfile.             */
 
   int extra_fd;              /* A place for the caller to store an FD.  */
 
-  /* An optional info callback - see dotlock_set_info_cb.             */
-  int (*info_cb)(dotlock_t, void *,
-                 enum dotlock_reasons reason,
-                 const char *,...);
-  void *info_cb_value;
-
-
 #ifdef HAVE_DOSISH_SYSTEM
   HANDLE lockhd;       /* The W32 handle of the lock file.      */
 #else /*!HAVE_DOSISH_SYSTEM */
@@ -442,8 +437,6 @@ static int never_lock;
 
 
 #ifdef HAVE_DOSISH_SYSTEM
-/* FIXME: For use in GnuPG this can be replaced by
- *        gnupg_w32_set_errno.  */
 static int
 map_w32_to_errno (DWORD w32_err)
 {
@@ -478,21 +471,6 @@ map_w32_to_errno (DWORD w32_err)
 }
 #endif /*HAVE_DOSISH_SYSTEM*/
 
-
-#ifdef HAVE_W32_SYSTEM
-static int
-any8bitchar (const char *string)
-{
-  if (string)
-    for ( ; *string; string++)
-      if ((*string & 0x80))
-        return 1;
-  return 0;
-}
-#endif /*HAVE_W32_SYSTEM*/
-
-
-
 
 /* Entirely disable all locking.  This function should be called
    before any locking is done.  It may be called right at startup of
@@ -531,7 +509,7 @@ maybe_deadlock (dotlock_t h)
    has been created on the same node. */
 #ifdef HAVE_POSIX_SYSTEM
 static int
-read_lockfile (dotlock_t h, int *same_node, int *r_fd)
+read_lockfile (dotlock_t h, int *same_node )
 {
   char buffer_space[10+1+70+1]; /* 70 is just an estimated value; node
                                    names are usually shorter. */
@@ -555,15 +533,8 @@ read_lockfile (dotlock_t h, int *same_node, int *r_fd)
   if ( (fd = open (h->lockname, O_RDONLY)) == -1 )
     {
       int e = errno;
-      if (errno != ENOENT)
-        {
-          my_info_2 ("error opening lockfile '%s': %s\n",
-                     h->lockname, strerror(errno) );
-          if (h->info_cb)
-            h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                        "error opening lockfile '%s': %s\n",
-                        h->lockname, strerror (errno) );
-        }
+      my_info_2 ("error opening lockfile '%s': %s\n",
+                 h->lockname, strerror(errno) );
       if (buffer != buffer_space)
         xfree (buffer);
       my_set_errno (e); /* Need to return ERRNO here. */
@@ -581,10 +552,6 @@ read_lockfile (dotlock_t h, int *same_node, int *r_fd)
         {
           int e = errno;
           my_info_1 ("error reading lockfile '%s'\n", h->lockname );
-          if (h->info_cb)
-            h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                        "error reading lockfile '%s': %s\n",
-                        h->lockname, strerror (errno) );
           close (fd);
           if (buffer != buffer_space)
             xfree (buffer);
@@ -595,18 +562,11 @@ read_lockfile (dotlock_t h, int *same_node, int *r_fd)
       nread += res;
     }
   while (res && nread != expected_len);
-
-  if (r_fd)
-    *r_fd = fd;
-  else
-    close(fd);
+  close(fd);
 
   if (nread < 11)
     {
       my_info_1 ("invalid size of lockfile '%s'\n", h->lockname);
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_INV_FILE,
-                    "invalid size of lockfile '%s'\n", h->lockname);
       if (buffer != buffer_space)
         xfree (buffer);
       my_set_errno (EINVAL);
@@ -618,9 +578,6 @@ read_lockfile (dotlock_t h, int *same_node, int *r_fd)
       || !pid )
     {
       my_error_2 ("invalid pid %d in lockfile '%s'\n", pid, h->lockname);
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_INV_FILE,
-                    "invalid pid %d in lockfile '%s'\n", pid, h->lockname);
       if (buffer != buffer_space)
         xfree (buffer);
       my_set_errno (EINVAL);
@@ -682,80 +639,6 @@ use_hardlinks_p (const char *tname)
 
 
 #ifdef  HAVE_POSIX_SYSTEM
-static int
-dotlock_get_process_id (dotlock_t h)
-{
-  return h->by_parent? (int)getppid(): (int)getpid();
-}
-
-static int
-dotlock_detect_tname (dotlock_t h)
-{
-  struct stat sb;
-  DIR *dir;
-  char *dirname;
-  char *basename;
-  struct dirent *d;
-  int r;
-
-  if (stat (h->lockname, &sb))
-    return -1;
-
-  basename = make_basename (h->lockname, NULL);
-  dirname = make_dirname (h->lockname);
-
-  dir = opendir (dirname);
-  if (dir == NULL)
-    {
-      xfree (basename);
-      xfree (dirname);
-      return -1;
-    }
-
-  while ((d = readdir (dir)))
-    if (sb.st_ino == d->d_ino && strcmp (d->d_name, basename))
-      break;
-
-  if (d)
-    {
-      int len = strlen (h->tname);
-      int dlen = strlen (d->d_name);
-      const char *tname_path;
-
-      if (dlen > len)
-        {
-          xfree (basename);
-          xfree (dirname);
-          return -1;
-        }
-
-      strcpy (stpcpy (stpcpy (h->tname, dirname), DIRSEP_S), d->d_name);
-      h->use_o_excl = 0;
-      tname_path = strchr (h->tname + strlen (dirname) + 2, '.');
-      if (!tname_path)
-        {
-          xfree (basename);
-          xfree (dirname);
-          return -1;
-        }
-      h->nodename_off = tname_path - h->tname + 1;
-    }
-  else
-    h->use_o_excl = 1;
-
-  r = closedir (dir);
-  if (r)
-    {
-      xfree (basename);
-      xfree (dirname);
-      return r;
-    }
-
-  xfree (basename);
-  xfree (dirname);
-  return 0;
-}
-
 /* Locking core for Unix.  It used a temporary file and the link
    system call to make locking an atomic operation. */
 static dotlock_t
@@ -768,10 +651,8 @@ dotlock_create_unix (dotlock_t h, const char *file_to_lock)
   int dirpartlen;
   struct utsname utsbuf;
   size_t tnamelen;
-  int pid;
 
-  pid = dotlock_get_process_id (h);
-  snprintf (pidstr, sizeof pidstr, "%10d\n", pid);
+  snprintf (pidstr, sizeof pidstr, "%10d\n", (int)getpid() );
 
   /* Create a temporary file. */
   if ( uname ( &utsbuf ) )
@@ -805,17 +686,10 @@ dotlock_create_unix (dotlock_t h, const char *file_to_lock)
     }
   h->nodename_len = strlen (nodename);
 
-  if (h->no_write)
-    {
-      memset (h->tname, '_', tnamelen);
-      h->tname[tnamelen] = 0;
-      goto skip_write;
-    }
-
   snprintf (h->tname, tnamelen, "%.*s/.#lk%p.", dirpartlen, dirpart, h );
   h->nodename_off = strlen (h->tname);
   snprintf (h->tname+h->nodename_off, tnamelen - h->nodename_off,
-           "%s.%d", nodename, pid);
+           "%s.%d", nodename, (int)getpid ());
 
   do
     {
@@ -832,10 +706,6 @@ dotlock_create_unix (dotlock_t h, const char *file_to_lock)
       UNLOCK_all_lockfiles ();
       my_error_2 (_("failed to create temporary file '%s': %s\n"),
                   h->tname, strerror (errno));
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_WAITING,
-                    _("failed to create temporary file '%s': %s\n"),
-                    h->tname, strerror (errno));
       xfree (h->tname);
       xfree (h);
       my_set_errno (saveerrno);
@@ -869,16 +739,11 @@ dotlock_create_unix (dotlock_t h, const char *file_to_lock)
         int saveerrno = errno;
         my_error_2 ("can't check whether hardlinks are supported for '%s': %s\n"
                     , h->tname, strerror (saveerrno));
-        if (h->info_cb)
-          h->info_cb (h, h->info_cb_value, DOTLOCK_CONFIG_TEST,
-                   "can't check whether hardlinks are supported for '%s': %s\n"
-                   , h->tname, strerror (saveerrno));
         my_set_errno (saveerrno);
       }
       goto write_failed;
     }
 
- skip_write:
   h->lockname = xtrymalloc (strlen (file_to_lock) + 6 );
   if (!h->lockname)
     {
@@ -893,20 +758,8 @@ dotlock_create_unix (dotlock_t h, const char *file_to_lock)
     }
   strcpy (stpcpy (h->lockname, file_to_lock), EXTSEP_S "lock");
   UNLOCK_all_lockfiles ();
-
-  if (h->no_write)
-    {
-      if (dotlock_detect_tname (h) < 0)
-        {
-          xfree (h->lockname);
-          xfree (h->tname);
-          xfree (h);
-          my_set_errno (EACCES);
-          return NULL;
-        }
-
-      h->locked = 1;
-    }
+  if (h->use_o_excl)
+    my_debug_1 ("locking for '%s' done via O_EXCL\n", h->lockname);
 
   return h;
 
@@ -916,11 +769,6 @@ dotlock_create_unix (dotlock_t h, const char *file_to_lock)
     all_lockfiles = h->next;
     UNLOCK_all_lockfiles ();
     my_error_2 (_("error writing to '%s': %s\n"), h->tname, strerror (errno));
-    if (h->info_cb)
-      h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                  _("error writing to '%s': %s\n"),
-                  h->tname, strerror (errno));
-
     if ( fd != -1 )
       close (fd);
     unlink (h->tname);
@@ -946,7 +794,7 @@ dotlock_create_w32 (dotlock_t h, const char *file_to_lock)
   h->next = all_lockfiles;
   all_lockfiles = h;
 
-  h->lockname = strconcat (file_to_lock, EXTSEP_S "lock", NULL);
+  h->lockname = xtrymalloc ( strlen (file_to_lock) + 6 );
   if (!h->lockname)
     {
       all_lockfiles = h->next;
@@ -954,6 +802,7 @@ dotlock_create_w32 (dotlock_t h, const char *file_to_lock)
       xfree (h);
       return NULL;
     }
+  strcpy (stpcpy(h->lockname, file_to_lock), EXTSEP_S "lock");
 
   /* If would be nice if we would use the FILE_FLAG_DELETE_ON_CLOSE
      along with FILE_SHARE_DELETE but that does not work due to a race
@@ -963,34 +812,31 @@ dotlock_create_w32 (dotlock_t h, const char *file_to_lock)
      reasons why a lock file can't be created and thus the process
      would not stop as expected but spin until Windows crashes.  Our
      solution is to keep the lock file open; that does not harm. */
-  if (any8bitchar (h->lockname))
-    {
-      wchar_t *wname = utf8_to_wchar (h->lockname);
+  {
+#ifdef HAVE_W32CE_SYSTEM
+    wchar_t *wname = utf8_to_wchar (h->lockname);
 
-      if (wname)
-        h->lockhd = CreateFileW (wname,
-                                 GENERIC_READ|GENERIC_WRITE,
-                                 FILE_SHARE_READ|FILE_SHARE_WRITE,
-                                 NULL, OPEN_ALWAYS, 0, NULL);
-      else
-        h->lockhd = INVALID_HANDLE_VALUE;
-      xfree (wname);
-    }
-  else
-    h->lockhd = CreateFileA (h->lockname,
-                             GENERIC_READ|GENERIC_WRITE,
-                             FILE_SHARE_READ|FILE_SHARE_WRITE,
-                             NULL, OPEN_ALWAYS, 0, NULL);
+    if (wname)
+      h->lockhd = CreateFile (wname,
+                              GENERIC_READ|GENERIC_WRITE,
+                              FILE_SHARE_READ|FILE_SHARE_WRITE,
+                              NULL, OPEN_ALWAYS, 0, NULL);
+    else
+      h->lockhd = INVALID_HANDLE_VALUE;
+    xfree (wname);
+#else
+    h->lockhd = CreateFile (h->lockname,
+                            GENERIC_READ|GENERIC_WRITE,
+                            FILE_SHARE_READ|FILE_SHARE_WRITE,
+                            NULL, OPEN_ALWAYS, 0, NULL);
+#endif
+  }
   if (h->lockhd == INVALID_HANDLE_VALUE)
     {
       int saveerrno = map_w32_to_errno (GetLastError ());
       all_lockfiles = h->next;
       UNLOCK_all_lockfiles ();
       my_error_2 (_("can't create '%s': %s\n"), h->lockname, w32_strerror (-1));
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                    _("can't create '%s': %s\n"),
-                    h->lockname, w32_strerror (-1));
       xfree (h->lockname);
       xfree (h);
       my_set_errno (saveerrno);
@@ -1015,15 +861,7 @@ dotlock_create_w32 (dotlock_t h, const char *file_to_lock)
    POSIX systems a temporary file ".#lk.<hostname>.pid[.threadid] is
    used.
 
-   FLAGS may include DOTLOCK_PREPARE_CREATE bit, which only allocates
-   the handle and requires a further call to dotlock_finish_create.
-   This can be used to set a callback between these calls.
-
-   FLAGS may include DOTLOCK_LOCK_BY_PARENT bit, when it's the parent
-   process controlling the lock.  This is used by dotlock util.
-
-   FLAGS may include DOTLOCK_LOCKED bit, when it should not create the
-   lockfile, but to unlock.  This is used by dotlock util.
+   FLAGS must be 0.
 
    The function returns an new handle which needs to be released using
    destroy_dotlock but gets also released at the termination of the
@@ -1035,13 +873,8 @@ dotlock_create (const char *file_to_lock, unsigned int flags)
 {
   static int initialized;
   dotlock_t h;
-#ifndef HAVE_DOSISH_SYSTEM
-  int by_parent = 0;
-  int no_write = 0;
-#endif
 
-  if ( !(flags & DOTLOCK_LOCK_BY_PARENT)
-       && !initialized )
+  if ( !initialized )
     {
       atexit (dotlock_remove_lockfiles);
       initialized = 1;
@@ -1050,15 +883,7 @@ dotlock_create (const char *file_to_lock, unsigned int flags)
   if ( !file_to_lock )
     return NULL;  /* Only initialization was requested.  */
 
-#ifndef HAVE_DOSISH_SYSTEM
-  if ((flags & DOTLOCK_LOCK_BY_PARENT) || (flags & DOTLOCK_LOCKED))
-    {
-      by_parent = !!(flags & DOTLOCK_LOCK_BY_PARENT);
-      no_write = !!(flags & DOTLOCK_LOCKED);
-      flags &= ~(DOTLOCK_LOCK_BY_PARENT | DOTLOCK_LOCKED);
-    }
-#endif
-  if ((flags & ~DOTLOCK_PREPARE_CREATE))
+  if (flags)
     {
       my_set_errno (EINVAL);
       return NULL;
@@ -1068,10 +893,6 @@ dotlock_create (const char *file_to_lock, unsigned int flags)
   if (!h)
     return NULL;
   h->extra_fd = -1;
-#ifndef HAVE_DOSISH_SYSTEM
-  h->by_parent = by_parent;
-  h->no_write = no_write;
-#endif
 
   if (never_lock)
     {
@@ -1083,24 +904,6 @@ dotlock_create (const char *file_to_lock, unsigned int flags)
       return h;
     }
 
-  if ((flags & DOTLOCK_PREPARE_CREATE))
-    return h;
-  else
-    return dotlock_finish_create (h, file_to_lock);
-}
-
-
-/* This function may be used along with dotlock_create (file_name,
- * DOTLOCK_PREPARE_CREATE) to finish the creation call. The given
- * filename shall be the same as passed to dotlock_create.  On success
- * the same handle H is returned, on error NULL is returned and H is
- * released.  */
-dotlock_t
-dotlock_finish_create (dotlock_t h, const char *file_to_lock)
-{
-  if (!h || !file_to_lock)
-    return NULL;
-
 #ifdef HAVE_DOSISH_SYSTEM
   return dotlock_create_w32 (h, file_to_lock);
 #else /*!HAVE_DOSISH_SYSTEM */
@@ -1127,24 +930,6 @@ dotlock_get_fd (dotlock_t h)
 }
 
 
-/* Set a callback function for info diagnostics.  The callback
- * function CB is called with the handle, the opaque value OPAQUE, a
- * reason code, and a format string with its arguments.  The callback
- * shall return 0 to continue operation or true in which case the
- * current function will be terminated with an error.  */
-void
-dotlock_set_info_cb (dotlock_t h,
-                     int (*cb)(dotlock_t, void *,
-                               enum dotlock_reasons reason,
-                               const char *,...),
-                     void *opaque)
-{
-  h->info_cb = cb;
-  h->info_cb_value = opaque;
-}
-
-
-
 
 #ifdef HAVE_POSIX_SYSTEM
 /* Unix specific code of destroy_dotlock.  */
@@ -1155,6 +940,7 @@ dotlock_destroy_unix (dotlock_t h)
     unlink (h->lockname);
   if (h->tname && !h->use_o_excl)
     unlink (h->tname);
+  xfree (h->tname);
 }
 #endif /*HAVE_POSIX_SYSTEM*/
 
@@ -1200,74 +986,19 @@ dotlock_destroy (dotlock_t h)
   UNLOCK_all_lockfiles ();
 
   /* Then destroy the lock. */
-  if (!h->disable
-      && (!h->by_parent || h->no_write))
+  if (!h->disable)
     {
-      /* NOTE: under the condition of (by_parent && !no_write),
-         it doesn't come here.  So, the lock file remains.  */
 #ifdef HAVE_DOSISH_SYSTEM
       dotlock_destroy_w32 (h);
 #else /* !HAVE_DOSISH_SYSTEM */
       dotlock_destroy_unix (h);
 #endif /* HAVE_DOSISH_SYSTEM */
+      xfree (h->lockname);
     }
-
-#ifdef HAVE_POSIX_SYSTEM
-  /* When DOTLOCK_LOCK_BY_PARENT and lock fails,
-     the temporary file created should be removed.  */
-  if (h->by_parent && !h->no_write && !h->locked)
-    if (h->tname && !h->use_o_excl)
-      unlink (h->tname);
-
-  xfree (h->tname);
-#endif
-  xfree (h->lockname);
   xfree(h);
 }
 
 
-/* Return true if H has been taken.  */
-int
-dotlock_is_locked (dotlock_t h)
-{
-  return h && !!h->locked;
-}
-
-
-/* Return the next interval to wait.  WTIME and TIMEOUT are pointers
- * to the current state and are updated by this function.  The
- * returned value might be different from the value of WTIME.  */
-static int
-next_wait_interval (int *wtime, long *timeout)
-{
-  int result;
-
-  /* Wait until lock has been released.  We use retry intervals of 4,
-   * 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 512, 1024, 2048ms, and
-   * so on.  If wait-forever was requested we add a small random value
-   * to have different timeouts per process. */
-  if (!*wtime)
-    *wtime = 4;
-  else if (*wtime < 2048)
-    *wtime *= 2;
-  else
-    *wtime = 512;
-
-  result = *wtime;
-  if (*wtime > 8 && *timeout < 0)
-    result += ((unsigned int)getpid() % 37);
-
-  if (*timeout > 0)
-    {
-      if (result > *timeout)
-        result = *timeout;
-      *timeout -= result;
-    }
-
-  return result;
-}
-
-
 
 #ifdef HAVE_POSIX_SYSTEM
 /* Unix specific code of make_dotlock.  Returns 0 on success and -1 on
@@ -1276,7 +1007,6 @@ static int
 dotlock_take_unix (dotlock_t h, long timeout)
 {
   int wtime = 0;
-  int timedout = 0;
   int sumtime = 0;
   int pid;
   int lastpid = -1;
@@ -1284,12 +1014,13 @@ dotlock_take_unix (dotlock_t h, long timeout)
   const char *maybe_dead="";
   int same_node;
   int saveerrno;
-  int fd;
 
  again:
   if (h->use_o_excl)
     {
       /* No hardlink support - use open(O_EXCL).  */
+      int fd;
+
       do
         {
           my_set_errno (0);
@@ -1305,10 +1036,6 @@ dotlock_take_unix (dotlock_t h, long timeout)
           saveerrno = errno;
           my_error_2 ("lock not made: open(O_EXCL) of '%s' failed: %s\n",
                       h->lockname, strerror (saveerrno));
-          if (h->info_cb)
-            h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                        "lock not made: open(O_EXCL) of '%s' failed: %s\n",
-                        h->lockname, strerror (saveerrno));
           my_set_errno (saveerrno);
           return -1;
         }
@@ -1316,8 +1043,7 @@ dotlock_take_unix (dotlock_t h, long timeout)
         {
           char pidstr[16];
 
-          snprintf (pidstr, sizeof pidstr, "%10d\n",
-                    dotlock_get_process_id (h));
+          snprintf (pidstr, sizeof pidstr, "%10d\n", (int)getpid());
           if (write (fd, pidstr, 11 ) == 11
               && write (fd, h->tname + h->nodename_off,h->nodename_len)
               == h->nodename_len
@@ -1331,10 +1057,6 @@ dotlock_take_unix (dotlock_t h, long timeout)
           saveerrno = errno;
           my_error_2 ("lock not made: writing to '%s' failed: %s\n",
                       h->lockname, strerror (errno));
-          if (h->info_cb)
-            h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                        "lock not made: writing to '%s' failed: %s\n",
-                        h->lockname, strerror (errno));
           close (fd);
           unlink (h->lockname);
           my_set_errno (saveerrno);
@@ -1353,10 +1075,6 @@ dotlock_take_unix (dotlock_t h, long timeout)
           saveerrno = errno;
           my_error_1 ("lock not made: Oops: stat of tmp file failed: %s\n",
                       strerror (errno));
-          if (h->info_cb)
-            h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                        "lock not made: Oops: stat of tmp file failed: %s\n",
-                        strerror (errno));
           /* In theory this might be a severe error: It is possible
              that link succeeded but stat failed due to changed
              permissions.  We can't do anything about it, though.  */
@@ -1372,74 +1090,34 @@ dotlock_take_unix (dotlock_t h, long timeout)
     }
 
   /* Check for stale lock files.  */
-  if ( (pid = read_lockfile (h, &same_node, &fd)) == -1 )
+  if ( (pid = read_lockfile (h, &same_node)) == -1 )
     {
       if ( errno != ENOENT )
         {
           saveerrno = errno;
           my_info_0 ("cannot read lockfile\n");
-          if (h->info_cb)
-            h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                        "cannot read lockfile\n");
           my_set_errno (saveerrno);
           return -1;
         }
       my_info_0 ("lockfile disappeared\n");
       goto again;
     }
-  else if ( (pid == dotlock_get_process_id (h) && same_node && !h->by_parent)
-            || (same_node && kill (pid, 0) && errno == ESRCH) )
+  else if ( pid == getpid() && same_node )
     {
-      /* Stale lockfile is detected. */
-      struct stat sb;
-
-      /* Check if it's unlocked during examining the lockfile.  */
-      if (fstat (fd, &sb) || sb.st_nlink == 0)
-        {
-          /* It's gone already by another process.  */
-          close (fd);
-          goto again;
-        }
-
-      /*
-       * Here, although it's quite _rare_, we have a race condition.
-       *
-       * When multiple processes race on a stale lockfile, detecting
-       * AND removing should be done atomically.  That is, to work
-       * correctly, the file to be removed should be the one which is
-       * examined for detection.
-       *
-       * But, when it's not atomic, consider the case for us where it
-       * takes some time between the detection and the removal of the
-       * lockfile.
-       *
-       * In this situation, it is possible that the file which was
-       * detected as stale is already removed by another process and
-       * then new lockfile is created (by that process or other one).
-       *
-       * And it is newly created valid lockfile which is going to be
-       * removed by us.
-       *
-       * Consider this long comment as it expresses possible (long)
-       * time between fstat above and unlink below; Meanwhile, the
-       * lockfile in question may be removed and there may be new
-       * valid one.
-       *
-       * In short, when you see the message of removing stale lockfile
-       * when there are multiple processes for the work, there is
-       * (very) little possibility something went wrong.
-       */
-
-      unlink (h->lockname);
+      my_info_0 ("Oops: lock already held by us\n");
+      h->locked = 1;
+      return 0; /* okay */
+    }
+  else if ( same_node && kill (pid, 0) && errno == ESRCH )
+    {
+      /* Note: It is unlikley that we get a race here unless a pid is
+         reused too fast or a new process with the same pid as the one
+         of the stale file tries to lock right at the same time as we.  */
       my_info_1 (_("removing stale lockfile (created by %d)\n"), pid);
-      close (fd);
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_STALE_REMOVED,
-                    _("removing stale lockfile (created by %d)\n"), pid);
+      unlink (h->lockname);
       goto again;
     }
 
-  close (fd);
   if (lastpid == -1)
     lastpid = pid;
   ownerchanged = (pid != lastpid);
@@ -1447,39 +1125,42 @@ dotlock_take_unix (dotlock_t h, long timeout)
   if (timeout)
     {
       struct timeval tv;
-      int wtimereal;
 
-      if (ownerchanged)
-        wtime = 0;  /* Reset because owner chnaged.  */
+      /* Wait until lock has been released.  We use increasing retry
+         intervals of 50ms, 100ms, 200ms, 400ms, 800ms, 2s, 4s and 8s
+         but reset it if the lock owner meanwhile changed.  */
+      if (!wtime || ownerchanged)
+        wtime = 50;
+      else if (wtime < 800)
+        wtime *= 2;
+      else if (wtime == 800)
+        wtime = 2000;
+      else if (wtime < 8000)
+        wtime *= 2;
 
-      wtimereal = next_wait_interval (&wtime, &timeout);
-      if (!timeout)
-        timedout = 1;  /* remember.  */
+      if (timeout > 0)
+        {
+          if (wtime > timeout)
+            wtime = timeout;
+          timeout -= wtime;
+        }
 
-      sumtime += wtimereal;
+      sumtime += wtime;
       if (sumtime >= 1500)
         {
           sumtime = 0;
           my_info_3 (_("waiting for lock (held by %d%s) %s...\n"),
                      pid, maybe_dead, maybe_deadlock(h)? _("(deadlock?) "):"");
-          if (h->info_cb
-              && h->info_cb (h, h->info_cb_value, DOTLOCK_WAITING,
-                             _("waiting for lock (held by %d%s) %s...\n"),
-                             pid, maybe_dead,
-                             maybe_deadlock(h)? _("(deadlock?) "):""))
-            {
-              my_set_errno (ECANCELED);
-              return -1;
-            }
         }
 
-      tv.tv_sec = wtimereal / 1000;
-      tv.tv_usec = (wtimereal % 1000) * 1000;
+
+      tv.tv_sec = wtime / 1000;
+      tv.tv_usec = (wtime % 1000) * 1000;
       select (0, NULL, NULL, NULL, &tv);
       goto again;
     }
 
-  my_set_errno (timedout? ETIMEDOUT : EACCES);
+  my_set_errno (EACCES);
   return -1;
 }
 #endif /*HAVE_POSIX_SYSTEM*/
@@ -1492,7 +1173,6 @@ static int
 dotlock_take_w32 (dotlock_t h, long timeout)
 {
   int wtime = 0;
-  int timedout = 0;
   int w32err;
   OVERLAPPED ovl;
 
@@ -1511,46 +1191,45 @@ dotlock_take_w32 (dotlock_t h, long timeout)
     {
       my_error_2 (_("lock '%s' not made: %s\n"),
                   h->lockname, w32_strerror (w32err));
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                    _("lock '%s' not made: %s\n"),
-                    h->lockname, w32_strerror (w32err));
-      _set_errno (map_w32_to_errno (w32err));
+      my_set_errno (map_w32_to_errno (w32err));
       return -1;
     }
 
   if (timeout)
     {
-      int wtimereal;
+      /* Wait until lock has been released.  We use retry intervals of
+         50ms, 100ms, 200ms, 400ms, 800ms, 2s, 4s and 8s.  */
+      if (!wtime)
+        wtime = 50;
+      else if (wtime < 800)
+        wtime *= 2;
+      else if (wtime == 800)
+        wtime = 2000;
+      else if (wtime < 8000)
+        wtime *= 2;
 
-      wtimereal = next_wait_interval (&wtime, &timeout);
-      if (!timeout)
-        timedout = 1;  /* remember.  */
-
-      if (wtime >= 800)
+      if (timeout > 0)
         {
-          my_info_1 (_("waiting for lock %s...\n"), h->lockname);
-          if (h->info_cb
-              && h->info_cb (h, h->info_cb_value, DOTLOCK_WAITING,
-                             _("waiting for lock %s...\n"), h->lockname))
-            {
-              my_set_errno (ECANCELED);
-              return -1;
-            }
+          if (wtime > timeout)
+            wtime = timeout;
+          timeout -= wtime;
         }
 
-      Sleep (wtimereal);
+      if (wtime >= 800)
+        my_info_1 (_("waiting for lock %s...\n"), h->lockname);
+
+      Sleep (wtime);
       goto again;
     }
 
-  my_set_errno (timedout? ETIMEDOUT : EACCES);
+  my_set_errno (EACCES);
   return -1;
 }
 #endif /*HAVE_DOSISH_SYSTEM*/
 
 
 /* Take a lock on H.  A value of 0 for TIMEOUT returns immediately if
-   the lock can't be taken, -1 waits forever (hopefully not), other
+   the lock can't be taked, -1 waits forever (hopefully not), other
    values wait for TIMEOUT milliseconds.  Returns: 0 on success  */
 int
 dotlock_take (dotlock_t h, long timeout)
@@ -1585,23 +1264,17 @@ dotlock_release_unix (dotlock_t h)
   int pid, same_node;
   int saveerrno;
 
-  pid = read_lockfile (h, &same_node, NULL);
+  pid = read_lockfile (h, &same_node);
   if ( pid == -1 )
     {
       saveerrno = errno;
       my_error_0 ("release_dotlock: lockfile error\n");
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                    "release_dotlock: lockfile error\n");
       my_set_errno (saveerrno);
       return -1;
     }
-  if ( pid != dotlock_get_process_id (h) || !same_node )
+  if ( pid != getpid() || !same_node )
     {
       my_error_1 ("release_dotlock: not our lock (pid=%d)\n", pid);
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_CONFLICT,
-                    "release_dotlock: not our lock (pid=%d)\n", pid);
       my_set_errno (EACCES);
       return -1;
     }
@@ -1611,10 +1284,6 @@ dotlock_release_unix (dotlock_t h)
       saveerrno = errno;
       my_error_1 ("release_dotlock: error removing lockfile '%s'\n",
                   h->lockname);
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                    "release_dotlock: error removing lockfile '%s'\n",
-                    h->lockname);
       my_set_errno (saveerrno);
       return -1;
     }
@@ -1635,15 +1304,10 @@ dotlock_release_w32 (dotlock_t h)
   memset (&ovl, 0, sizeof ovl);
   if (!UnlockFileEx (h->lockhd, 0, 1, 0, &ovl))
     {
-      int ec = (int)GetLastError ();
-
+      int saveerrno = map_w32_to_errno (GetLastError ());
       my_error_2 ("release_dotlock: error removing lockfile '%s': %s\n",
-                  h->lockname, w32_strerror (ec));
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_FILE_ERROR,
-                    "release_dotlock: error removing lockfile '%s': %s\n",
-                    h->lockname, w32_strerror (ec));
-      my_set_errno (map_w32_to_errno (ec));
+                  h->lockname, w32_strerror (-1));
+      my_set_errno (saveerrno);
       return -1;
     }
 
@@ -1674,9 +1338,6 @@ dotlock_release (dotlock_t h)
   if ( !h->locked )
     {
       my_debug_1 ("Oops, '%s' is not locked\n", h->lockname);
-      if (h->info_cb)
-        h->info_cb (h, h->info_cb_value, DOTLOCK_NOT_LOCKED,
-                    "Oops, '%s' is not locked\n", h->lockname);
       return 0;
     }
 
