@@ -1,6 +1,6 @@
 /* exechelp.c - Fork and exec helpers for POSIX
- * Copyright (C) 2004, 2007-2009, 2010 Free Software Foundation, Inc.
- * Copyright (C) 2004, 2006-2012, 2014-2017 g10 Code GmbH
+ * Copyright (C) 2004, 2007, 2008, 2009,
+ *               2010 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -26,12 +26,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <https://www.gnu.org/licenses/>.
- * SPDX-License-Identifier: (LGPL-3.0+ OR GPL-2.0+)
  */
 
 #include <config.h>
 
-#if defined(HAVE_W32_SYSTEM)
+#if defined(HAVE_W32_SYSTEM) || defined (HAVE_W32CE_SYSTEM)
 #error This code is only used on POSIX
 #endif
 
@@ -277,21 +276,16 @@ get_all_open_fds (void)
 static void
 do_exec (const char *pgmname, const char *argv[],
          int fd_in, int fd_out, int fd_err,
-         int *except, unsigned int flags)
+         int *except, void (*preexec)(void) )
 {
   char **arg_list;
   int i, j;
   int fds[3];
-  int nodevnull[3];
 
   fds[0] = fd_in;
   fds[1] = fd_out;
   fds[2] = fd_err;
 
-  nodevnull[0] = !!(flags & GNUPG_SPAWN_KEEP_STDIN);
-  nodevnull[1] = !!(flags & GNUPG_SPAWN_KEEP_STDOUT);
-  nodevnull[2] = !!(flags & GNUPG_SPAWN_KEEP_STDERR);
-
   /* Create the command line argument array.  */
   i = 0;
   if (argv)
@@ -310,9 +304,7 @@ do_exec (const char *pgmname, const char *argv[],
   /* Assign /dev/null to unused FDs. */
   for (i=0; i <= 2; i++)
     {
-      if (nodevnull[i])
-        continue;
-      if (fds[i] == -1)
+      if (fds[i] == -1 )
         {
           fds[i] = open ("/dev/null", i? O_WRONLY : O_RDONLY);
           if (fds[i] == -1)
@@ -324,8 +316,6 @@ do_exec (const char *pgmname, const char *argv[],
   /* Connect the standard files.  */
   for (i=0; i <= 2; i++)
     {
-      if (nodevnull[i])
-        continue;
       if (fds[i] != i && dup2 (fds[i], i) == -1)
         log_fatal ("dup2 std%s failed: %s\n",
                    i==0?"in":i==1?"out":"err", strerror (errno));
@@ -334,6 +324,8 @@ do_exec (const char *pgmname, const char *argv[],
   /* Close all other files. */
   close_all_fds (3, except);
 
+  if (preexec)
+    preexec ();
   execv (pgmname, arg_list);
   /* No way to print anything, as we have closed all streams. */
   _exit (127);
@@ -423,19 +415,10 @@ gnupg_create_pipe (int filedes[2])
 }
 
 
-/* Close the end of a pipe.  */
-void
-gnupg_close_pipe (int fd)
-{
-  if (fd != -1)
-    close (fd);
-}
-
-
 /* Fork and exec the PGMNAME, see exechelp.h for details.  */
 gpg_error_t
 gnupg_spawn_process (const char *pgmname, const char *argv[],
-                     int *except, unsigned int flags,
+                     int *except, void (*preexec)(void), unsigned int flags,
                      estream_t *r_infp,
                      estream_t *r_outfp,
                      estream_t *r_errfp,
@@ -542,7 +525,7 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
       es_fclose (outfp);
       es_fclose (errfp);
       do_exec (pgmname, argv, inpipe[0], outpipe[1], errpipe[1],
-               except, flags);
+               except, preexec);
       /*NOTREACHED*/
     }
 
@@ -592,7 +575,7 @@ gnupg_spawn_process_fd (const char *pgmname, const char *argv[],
     {
       gcry_control (GCRYCTL_TERM_SECMEM);
       /* Run child. */
-      do_exec (pgmname, argv, infd, outfd, errfd, NULL, 0);
+      do_exec (pgmname, argv, infd, outfd, errfd, NULL, NULL);
       /*NOTREACHED*/
     }
 
@@ -739,13 +722,8 @@ gnupg_wait_processes (const char **pgmnames, pid_t *pids, size_t count,
     {
       int status = -1;
 
-      /* Skip invalid PID.  */
       if (pids[i] == (pid_t)(-1))
-        {
-          r_exitcodes[i] = -1;
-          left -= 1;
-          continue;
-        }
+        return my_error (GPG_ERR_INV_VALUE);
 
       /* See if there was a previously stored result for this pid.  */
       if (get_result (pids[i], &status))
@@ -857,15 +835,14 @@ gpg_error_t
 gnupg_spawn_process_detached (const char *pgmname, const char *argv[],
                               const char *envp[] )
 {
-  gpg_err_code_t ec;
   pid_t pid;
   int i;
 
   if (getuid() != geteuid())
     return my_error (GPG_ERR_BUG);
 
-  if ((ec = gnupg_access (pgmname, X_OK)))
-    return gpg_err_make (default_errsource, ec);
+  if (access (pgmname, X_OK))
+    return my_error_from_syserror ();
 
   pid = fork ();
   if (pid == (pid_t)(-1))
@@ -891,7 +868,7 @@ gnupg_spawn_process_detached (const char *pgmname, const char *argv[],
         for (i=0; envp[i]; i++)
           putenv (xstrdup (envp[i]));
 
-      do_exec (pgmname, argv, -1, -1, -1, NULL, 0);
+      do_exec (pgmname, argv, -1, -1, -1, NULL, NULL);
 
       /*NOTREACHED*/
     }
