@@ -30,19 +30,18 @@
 #include <config.h>
 
 #ifdef HAVE_W32_SYSTEM
-# if _WIN32_WINNT < 0x0600
-#   define _WIN32_WINNT 0x0600  /* Required for SetProcessDEPPolicy.  */
-# endif
 # ifdef HAVE_WINSOCK2_H
 #  include <winsock2.h>
 # endif
 # include <windows.h>
 #endif
+#ifdef HAVE_W32CE_SYSTEM
+# include <assuan.h> /* For _assuan_w32ce_finish_pipe. */
+#endif
 
 #include <gcrypt.h>
 #include "util.h"
 #include "i18n.h"
-#include "w32help.h"
 
 /* This object is used to register memory cleanup functions.
    Technically they are not needed but they can avoid frequent
@@ -69,11 +68,16 @@ static mem_cleanup_item_t mem_cleanup_list;
 gpg_err_source_t default_errsource = 0;
 
 
-
-#if HAVE_W32_SYSTEM
-static void prepare_w32_commandline (int *argcp, char ***argvp);
-#endif /*HAVE_W32_SYSTEM*/
-
+#ifdef HAVE_W32CE_SYSTEM
+static void parse_std_file_handles (int *argcp, char ***argvp);
+static void
+sleep_on_exit (void)
+{
+  /* The sshd on CE swallows some of the command output.  Sleeping a
+     while usually helps.  */
+  Sleep (400);
+}
+#endif /*HAVE_W32CE_SYSTEM*/
 
 
 static void
@@ -168,6 +172,14 @@ _init_common_subsystems (gpg_err_source_t errsource, int *argcp, char ***argvp)
   }
 #endif
 
+#ifdef HAVE_W32CE_SYSTEM
+  /* Register the sleep exit function before the estream init so that
+     the sleep will be called after the estream registered atexit
+     function which flushes the left open estream streams and in
+     particular es_stdout.  */
+  atexit (sleep_on_exit);
+#endif
+
   if (!gcry_check_version (NEED_LIBGCRYPT_VERSION))
     {
       log_fatal (_("%s is too old (need %s, have %s)\n"), "libgcrypt",
@@ -178,22 +190,13 @@ _init_common_subsystems (gpg_err_source_t errsource, int *argcp, char ***argvp)
   gpgrt_init ();
   gpgrt_set_alloc_func (gcry_realloc);
 
-
-#ifdef HAVE_W32_SYSTEM
-  /* We want gettext to always output UTF-8 and we put the console in
-   * utf-8 mode.  */
-  gettext_use_utf8 (1);
-  if (!SetConsoleCP (CP_UTF8) || !SetConsoleOutputCP (CP_UTF8))
-    {
-      /* Don't show the error if the program does not have a console.
-       * This is for example the case for daemons.  */
-      int rc = GetLastError ();
-      if (rc != ERROR_INVALID_HANDLE)
-        {
-          log_info ("SetConsoleCP failed: %s\n", w32_strerror (rc));
-          log_info ("Warning: Garbled console data possible\n");
-        }
-    }
+  /* Special hack for Windows CE: We extract some options from arg
+     to setup the standard handles.  */
+#ifdef HAVE_W32CE_SYSTEM
+  parse_std_file_handles (argcp, argvp);
+#else
+  (void)argcp;
+  (void)argvp;
 #endif
 
   /* Access the standard estreams as early as possible.  If we don't
@@ -207,98 +210,80 @@ _init_common_subsystems (gpg_err_source_t errsource, int *argcp, char ***argvp)
   }
 
   /* --version et al shall use estream as well.  */
-  gpgrt_set_usage_outfnc (writestring_via_estream);
-
-  /* Register our string mapper with gpgrt.  */
-  gpgrt_set_fixed_string_mapper (map_static_macro_string);
+  argparse_register_outfnc (writestring_via_estream);
 
   /* Logging shall use the standard socket directory as fallback.  */
   log_set_socket_dir_cb (gnupg_socketdir);
-
-#if HAVE_W32_SYSTEM
-  /* Make sure that Data Execution Prevention is enabled.  */
-  if (GetSystemDEPPolicy () >= 2)
-    {
-      DWORD flags;
-      BOOL perm;
-
-      if (!GetProcessDEPPolicy (GetCurrentProcess (), &flags, &perm))
-        log_info ("error getting DEP policy: %s\n",
-                  w32_strerror (GetLastError()));
-      else if (!(flags & PROCESS_DEP_ENABLE)
-               && !SetProcessDEPPolicy (PROCESS_DEP_ENABLE))
-        log_info ("Warning: Enabling DEP failed: %s (%d,%d)\n",
-                  w32_strerror (GetLastError ()), (int)flags, (int)perm);
-    }
-  /* On Windows we use our own parser for the command line
-   * so that we can return an array of utf-8 encoded strings.  */
-  prepare_w32_commandline (argcp, argvp);
-#else
-  (void)argcp;
-  (void)argvp;
-#endif
-
 }
 
 
 
-/* For Windows we need to parse the command line so that we can
- * provide an UTF-8 encoded argv.  If there is any Unicode character
- * we return a new array but if there is no Unicode character we do
- * nothing.  */
-#ifdef HAVE_W32_SYSTEM
+/* WindowsCE uses a very strange way of handling the standard streams.
+   There is a function SetStdioPath to associate a standard stream
+   with a file or a device but what we really want is to use pipes as
+   standard streams.  Despite that we implement pipes using a device,
+   we would have some limitations on the number of open pipes due to
+   the 3 character limit of device file name.  Thus we don't take this
+   path.  Another option would be to install a file system driver with
+   support for pipes; this would allow us to get rid of the device
+   name length limitation.  However, with GnuPG we can get away be
+   redefining the standard streams and passing the handles to be used
+   on the command line.  This has also the advantage that it makes
+   creating a process much easier and does not require the
+   SetStdioPath set and restore game.  The caller needs to pass the
+   rendezvous ids using up to three options:
+
+     -&S0=<rvid> -&S1=<rvid> -&S2=<rvid>
+
+   They are all optional but they must be the first arguments on the
+   command line.  Parsing stops as soon as an invalid option is found.
+   These rendezvous ids are then used to finish the pipe creation.*/
+#ifdef HAVE_W32CE_SYSTEM
 static void
-prepare_w32_commandline (int *r_argc, char ***r_argv)
+parse_std_file_handles (int *argcp, char ***argvp)
 {
-  const wchar_t *wcmdline, *ws;
-  char *cmdline;
-  int argc;
-  char **argv;
+  int argc = *argcp;
+  char **argv = *argvp;
   const char *s;
-  int i, globing, itemsalloced;
+  assuan_fd_t fd;
+  int i;
+  int fixup = 0;
 
-  s = gpgrt_strusage (95);
-  globing = (s && *s == '1');
+  if (!argc)
+    return;
 
-  wcmdline = GetCommandLineW ();
-  if (!wcmdline)
+  for (argc--, argv++; argc; argc--, argv++)
     {
-      log_error ("GetCommandLineW failed\n");
-      return;  /* Ooops.  */
+      s = *argv;
+      if (*s == '-' && s[1] == '&' && s[2] == 'S'
+          && (s[3] == '0' || s[3] == '1' || s[3] == '2')
+          && s[4] == '='
+          && (strchr ("-01234567890", s[5]) || !strcmp (s+5, "null")))
+        {
+          if (s[5] == 'n')
+            fd = ASSUAN_INVALID_FD;
+          else
+            fd = _assuan_w32ce_finish_pipe (atoi (s+5), s[3] != '0');
+          _es_set_std_fd (s[3] - '0', (int)fd);
+          fixup++;
+        }
+      else
+        break;
     }
 
-  if (!globing)
+  if (fixup)
     {
-      /* If globbing is not enabled we use our own parser only if
-       * there are any non-ASCII characters.  */
-      for (ws=wcmdline; *ws; ws++)
-        if (!iswascii (*ws))
-          break;
-      if (!*ws)
-        return;  /* No Unicode - return directly.  */
+      argc = *argcp;
+      argc -= fixup;
+      *argcp = argc;
+
+      argv = *argvp;
+      for (i=1; i < argc; i++)
+        argv[i] = argv[i + fixup];
+      for (; i < argc + fixup; i++)
+        argv[i] = NULL;
     }
 
-  cmdline = wchar_to_utf8 (wcmdline);
-  if (!cmdline)
-    {
-      log_error ("parsing command line failed: %s\n", strerror (errno));
-      return;  /* Ooops.  */
-    }
-  gpgrt_annotate_leaked_object (cmdline);
 
-  argv = w32_parse_commandline (cmdline, globing, &argc, &itemsalloced);
-  if (!argv)
-    {
-      log_error ("parsing command line failed: %s\n", "internal error");
-      return;  /* Ooops.  */
-    }
-  gpgrt_annotate_leaked_object (argv);
-  if (itemsalloced)
-    {
-      for (i=0; i < argc; i++)
-        gpgrt_annotate_leaked_object (argv[i]);
-    }
-  *r_argv = argv;
-  *r_argc = argc;
 }
-#endif /*HAVE_W32_SYSTEM*/
+#endif /*HAVE_W32CE_SYSTEM*/
