@@ -33,9 +33,6 @@
 #define NODFLG_SELKEY (1<<9)	/* Indicate the selected key.  */
 #define NODFLG_SELSIG (1<<10)	/* Indicate a selected signature.  */
 
-#define NODFLG_MARK_B (1<<11)   /* Temporary mark in key listing code.  */
-
-
 /*-- keyedit.c --*/
 void keyedit_menu (ctrl_t ctrl, const char *username, strlist_t locusr,
 		   strlist_t commands, int quiet, int seckey_check );
@@ -44,29 +41,20 @@ void keyedit_quick_adduid (ctrl_t ctrl, const char *username,
                            const char *newuid);
 void keyedit_quick_addkey (ctrl_t ctrl, const char *fpr, const char *algostr,
                            const char *usagestr, const char *expirestr);
-void keyedit_quick_addadsk (ctrl_t ctrl, const char *fpr, const char *adskfpr);
 void keyedit_quick_revuid (ctrl_t ctrl, const char *username,
                            const char *uidtorev);
 void keyedit_quick_sign (ctrl_t ctrl, const char *fpr,
                          strlist_t uids, strlist_t locusr, int local);
-void keyedit_quick_revsig (ctrl_t ctrl, const char *username,
-                           const char *sigtorev, strlist_t affected_uids);
 void keyedit_quick_set_expire (ctrl_t ctrl,
                                const char *fpr, const char *expirestr,
                                char **subkeyfprs);
 void keyedit_quick_set_primary (ctrl_t ctrl, const char *username,
                                 const char *primaryuid);
-void keyedit_quick_update_pref (ctrl_t ctrl, const char *username);
-void keyedit_quick_set_ownertrust (ctrl_t ctrl, const char *username,
-                                   const char *value);
-gpg_error_t append_adsk_to_key (ctrl_t ctrl, kbnode_t keyblock,
-                                PKT_public_key *adsk);
 void show_basic_key_info (ctrl_t ctrl, kbnode_t keyblock, int print_sec);
 int keyedit_print_one_sig (ctrl_t ctrl, estream_t fp,
                            int rc, kbnode_t keyblock,
 			   kbnode_t node, int *inv_sigs, int *no_key,
 			   int *oth_err, int is_selfsig,
 			   int print_without_key, int extended);
-void no_usable_encr_subkeys_warning (kbnode_t keyblock);
 
 #endif	/* GNUPG_G10_KEYEDIT_H */
