@@ -267,7 +267,7 @@ unknown_pubkey_warning (int algo)
      encryption/signing.  */
   if (pubkey_get_npkey (algo))
     {
-      if (opt.verbose)
+      if (opt.verbose && !glo_ctrl.silence_parse_warnings)
         {
           if (!pubkey_get_nsig (algo))
             log_info ("public key algorithm %s not suitable for %s\n",
@@ -282,7 +282,7 @@ unknown_pubkey_warning (int algo)
       algo &= 0xff;
       if (!unknown_pubkey_algos[algo])
         {
-          if (opt.verbose)
+          if (opt.verbose && !glo_ctrl.silence_parse_warnings)
             log_info (_("can't handle public key algorithm %d\n"), algo);
           unknown_pubkey_algos[algo] = 1;
         }
@@ -1548,6 +1548,24 @@ dump_sig_subpkt (int hashed, int type, int critical,
                     buffer[0] == 3 ? buffer[15] : buffer[2],
                     buffer[0] == 3 ? buffer[16] : buffer[3]);
       break;
+
+    case SIGSUBPKT_KEY_BLOCK:
+      es_fputs ("key-block: ", listfp);
+      if (length && buffer[0])
+        p = "[unknown reserved octet]";
+      else if (length < 50)  /* 50 is an arbitrary min. length.  */
+        p = "[invalid subpacket]";
+      else
+        {
+          /* estream_t fp; */
+          /* fp = es_fopen ("a.key-block", "wb"); */
+          /* log_assert (fp); */
+          /* es_fwrite ( buffer+1, length-1, 1, fp); */
+          /* es_fclose (fp); */
+          es_fprintf (listfp, "[%u octets]", (unsigned int)length-1);
+        }
+      break;
+
     default:
       if (type >= 100 && type <= 110)
 	p = "experimental / private subpacket";
@@ -1622,6 +1640,12 @@ parse_one_sig_subpkt (const byte * buffer, size_t n, int type)
       if (n != 2)
 	break;
       return 0;
+    case SIGSUBPKT_KEY_BLOCK:
+      if (n && buffer[0])
+        return -1; /* Unknown version - ignore.  */
+      if (n < 50)
+	break;  /* Definitely too short to carry a key block.  */
+      return 0;
     default:
       return 0;
     }
@@ -1641,7 +1665,7 @@ can_handle_critical_notation (const byte *name, size_t len)
     if (sl->flags == len && !memcmp (sl->d, name, len))
       return 1; /* Known */
 
-  if (opt.verbose)
+  if (opt.verbose && !glo_ctrl.silence_parse_warnings)
     {
       log_info(_("Unknown critical signature notation: ") );
       print_utf8_buffer (log_get_stream(), name, len);
@@ -1688,6 +1712,12 @@ can_handle_critical (const byte * buffer, size_t n, int type)
     case SIGSUBPKT_REVOC_REASON: /* At least we know about it.  */
       return 1;
 
+    case SIGSUBPKT_KEY_BLOCK:
+      if (n && !buffer[0])
+        return 1;
+      else
+        return 0;
+
     default:
       return 0;
     }
@@ -1761,7 +1791,7 @@ enum_sig_subpkt (const subpktarea_t * pktbuf, sigsubpkttype_t reqtype,
 		goto too_short;
 	      if (!can_handle_critical (buffer + 1, n - 1, type))
 		{
-		  if (opt.verbose)
+		  if (opt.verbose && !glo_ctrl.silence_parse_warnings)
 		    log_info (_("subpacket of type %d has "
 				"critical bit set\n"), type);
 		  if (start)
@@ -1812,14 +1842,14 @@ enum_sig_subpkt (const subpktarea_t * pktbuf, sigsubpkttype_t reqtype,
   return NULL;	/* End of packets; not found.  */
 
  too_short:
-  if (opt.verbose)
+  if (opt.verbose && !glo_ctrl.silence_parse_warnings)
     log_info ("buffer shorter than subpacket\n");
   if (start)
     *start = -1;
   return NULL;
 
  no_type_byte:
-  if (opt.verbose)
+  if (opt.verbose && !glo_ctrl.silence_parse_warnings)
     log_info ("type octet missing in subpacket\n");
   if (start)
     *start = -1;
@@ -2029,7 +2059,7 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
       if (p)
 	sig->timestamp = buf32_to_u32 (p);
       else if (!(sig->pubkey_algo >= 100 && sig->pubkey_algo <= 110)
-	       && opt.verbose)
+	       && opt.verbose && !glo_ctrl.silence_parse_warnings)
 	log_info ("signature packet without timestamp\n");
 
       p = parse_sig_subpkt2 (sig, SIGSUBPKT_ISSUER);
@@ -2039,7 +2069,7 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
 	  sig->keyid[1] = buf32_to_u32 (p + 4);
 	}
       else if (!(sig->pubkey_algo >= 100 && sig->pubkey_algo <= 110)
-	       && opt.verbose)
+	       && opt.verbose && !glo_ctrl.silence_parse_warnings)
 	log_info ("signature packet without keyid\n");
 
       p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_SIG_EXPIRE, NULL);
@@ -2079,6 +2109,10 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
       if (p)
 	sig->flags.notation = 1;
 
+      p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_KEY_BLOCK, NULL);
+      if (p)
+        sig->flags.key_block = 1;
+
       p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_REVOCABLE, NULL);
       if (p && *p == 0)
 	sig->flags.revocable = 0;
@@ -2286,7 +2320,9 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
     }
   else if (version == 2 || version == 3)
     {
-      if (opt.verbose > 1)
+      /* Not anymore supported since 2.1.  Use an older gpg version
+       * (i.e. gpg 1.4) to parse v3 packets.  */
+      if (opt.verbose > 1 && !glo_ctrl.silence_parse_warnings)
         log_info ("packet(%d) with obsolete version %d\n", pkttype, version);
       if (list_mode)
         es_fprintf (listfp, ":key packet: [obsolete version %d]\n", version);
@@ -2760,7 +2796,7 @@ parse_attribute_subpkts (PKT_user_id * uid)
   return count;
 
  too_short:
-  if (opt.verbose)
+  if (opt.verbose && !glo_ctrl.silence_parse_warnings)
     log_info ("buffer shorter than attribute subpacket\n");
   uid->attribs = attribs;
   uid->numattribs = count;
