@@ -48,6 +48,13 @@
 #define MY_O_BINARY  0
 #endif
 
+/* We use ERRNO despite that the cegcc provided open/read/write
+   functions don't set ERRNO - at least show that ERRNO does not make
+   sense.  */
+#ifdef HAVE_W32CE_SYSTEM
+#undef strerror
+#define strerror(a) ("[errno not available]")
+#endif
 
 /*
  * Yes, this is a very simple implementation. We should really
@@ -99,8 +106,8 @@ struct cmp_xdir_struct
 static char *db_name;
 
 /* The handle for locking the trustdb file and a counter to record how
- * often this lock has been taken.  That counter is required because
- * dotlock does not implement recursive locks.  */
+ * often this lock has been taken.  That counter is required becuase
+ * dotlock does not implemen recursive locks.  */
 static dotlock_t lockhandle;
 static unsigned int is_locked;
 
@@ -415,7 +422,7 @@ put_record_into_cache (ulong recno, const char *data)
 
 /* Return true if the cache is dirty.  */
 int
-tdbio_is_dirty (void)
+tdbio_is_dirty()
 {
   return cache_is_dirty;
 }
@@ -425,7 +432,7 @@ tdbio_is_dirty (void)
  * Flush the cache.  This cannot be used while in a transaction.
  */
 int
-tdbio_sync (void)
+tdbio_sync()
 {
     CACHE_CTRL r;
     int did_lock = 0;
@@ -582,7 +589,7 @@ tdbio_update_version_record (ctrl_t ctrl)
 
 /*
  * Create and write the trustdb version record.
- * This is called with the writelock active.
+ * This is called with the writelock activ.
  * Returns: 0 on success or an error code.
  */
 static int
@@ -674,7 +681,7 @@ tdbio_set_dbname (ctrl_t ctrl, const char *new_dbname,
    * trustdb.gpg.  This check is not required in theory, but it helps
    * in practice avoiding costly operations of preparing and taking
    * the lock.  */
-  if (!gnupg_stat (fname, &statbuf) && statbuf.st_size > 0)
+  if (!stat (fname, &statbuf) && statbuf.st_size > 0)
     {
       /* OK, we have the valid trustdb.gpg already.  */
       return 0;
@@ -704,25 +711,30 @@ tdbio_set_dbname (ctrl_t ctrl, const char *new_dbname,
   log_assert (p);
   save_slash = *p;
   *p = 0;
-  if (gnupg_access (fname, F_OK))
+  if (access (fname, F_OK))
     {
       try_make_homedir (fname);
-      if (gnupg_access (fname, F_OK))
+      if (access (fname, F_OK))
         log_fatal (_("%s: directory does not exist!\n"), fname);
     }
   *p = save_slash;
 
   take_write_lock ();
 
-  if (gnupg_access (fname, R_OK)
-      || gnupg_stat (fname, &statbuf)
-      || statbuf.st_size == 0)
+  if (access (fname, R_OK) || stat (fname, &statbuf) || statbuf.st_size == 0)
     {
-      estream_t fp;
+      FILE *fp;
       TRUSTREC rec;
       int rc;
       mode_t oldmask;
 
+#ifdef HAVE_W32CE_SYSTEM
+      /* We know how the cegcc implementation of access works ;-). */
+      if (GetLastError () == ERROR_FILE_NOT_FOUND)
+        gpg_err_set_errno (ENOENT);
+      else
+        gpg_err_set_errno (EIO);
+#endif /*HAVE_W32CE_SYSTEM*/
       if (errno && errno != ENOENT)
         log_fatal ( _("can't access '%s': %s\n"), fname, strerror (errno));
 
@@ -733,13 +745,13 @@ tdbio_set_dbname (ctrl_t ctrl, const char *new_dbname,
           gpg_err_set_errno (EPERM);
         }
       else
-        fp = es_fopen (fname, "wb");
+        fp = fopen (fname, "wb");
       umask(oldmask);
       if (!fp)
         log_fatal (_("can't create '%s': %s\n"), fname, strerror (errno));
-      es_fclose (fp);
+      fclose (fp);
 
-      db_fd = gnupg_open (db_name, O_RDWR | MY_O_BINARY, 0);
+      db_fd = open (db_name, O_RDWR | MY_O_BINARY);
       if (db_fd == -1)
         log_fatal (_("can't open '%s': %s\n"), db_name, strerror (errno));
 
@@ -765,7 +777,7 @@ tdbio_set_dbname (ctrl_t ctrl, const char *new_dbname,
  * Return the full name of the trustdb.
  */
 const char *
-tdbio_get_dbname (void)
+tdbio_get_dbname ()
 {
   return db_name;
 }
@@ -777,13 +789,29 @@ tdbio_get_dbname (void)
  * the trustdb handle (DB_FD) is guaranteed to be open.
  */
 static void
-open_db (void)
+open_db ()
 {
   TRUSTREC rec;
 
   log_assert( db_fd == -1 );
 
-  db_fd = gnupg_open (db_name, O_RDWR | MY_O_BINARY, 0);
+#ifdef HAVE_W32CE_SYSTEM
+  {
+    DWORD prevrc = 0;
+    wchar_t *wname = utf8_to_wchar (db_name);
+    if (wname)
+      {
+        db_fd = (int)CreateFile (wname, GENERIC_READ|GENERIC_WRITE,
+                                 FILE_SHARE_READ|FILE_SHARE_WRITE, NULL,
+                                 OPEN_EXISTING, 0, NULL);
+        xfree (wname);
+      }
+    if (db_fd == -1)
+      log_fatal ("can't open '%s': %d, %d\n", db_name,
+                 (int)prevrc, (int)GetLastError ());
+  }
+#else /*!HAVE_W32CE_SYSTEM*/
+  db_fd = open (db_name, O_RDWR | MY_O_BINARY );
   if (db_fd == -1 && (errno == EACCES
 #ifdef EROFS
                       || errno == EROFS
@@ -791,13 +819,13 @@ open_db (void)
                       )
       ) {
       /* Take care of read-only trustdbs.  */
-      db_fd = gnupg_open (db_name, O_RDONLY | MY_O_BINARY, 0);
+      db_fd = open (db_name, O_RDONLY | MY_O_BINARY );
       if (db_fd != -1 && !opt.quiet)
           log_info (_("Note: trustdb not writable\n"));
   }
   if ( db_fd == -1 )
     log_fatal( _("can't open '%s': %s\n"), db_name, strerror(errno) );
-
+#endif /*!HAVE_W32CE_SYSTEM*/
   register_secured_file (db_name);
 
   /* Read the version record. */
@@ -859,7 +887,7 @@ create_hashtable (ctrl_t ctrl, TRUSTREC *vr, int type)
  * Return: 1 for yes, 0 for no.
  */
 int
-tdbio_db_matches_options (void)
+tdbio_db_matches_options()
 {
   static int yes_no = -1;
 
@@ -916,7 +944,7 @@ tdbio_read_model (void)
  * problem the process is terminated.
  */
 ulong
-tdbio_read_nextcheck (void)
+tdbio_read_nextcheck ()
 {
   TRUSTREC vr;
   int rc;
@@ -1111,7 +1139,7 @@ upd_hashtable (ctrl_t ctrl, ulong table, byte *key, int keylen, ulong newrecnum)
 
               if (rec.r.hlst.next)
                 {
-                  /* read the next record of the list.  */
+                  /* read the next reord of the list.  */
                   rc = tdbio_read_record (rec.r.hlst.next, &rec, RECTYPE_HLST);
                   if (rc)
                     {
@@ -1456,19 +1484,16 @@ tdbio_dump_record (TRUSTREC *rec, estream_t fp)
       es_fprintf (fp, "trust ");
       for (i=0; i < 20; i++)
         es_fprintf (fp, "%02X", rec->r.trust.fingerprint[i]);
-      es_fprintf (fp, ", ot=%d, d=%d, vl=%lu, mo=%d, f=%02x\n",
-                  rec->r.trust.ownertrust,
-                  rec->r.trust.depth, rec->r.trust.validlist,
-                  rec->r.trust.min_ownertrust, rec->r.trust.flags);
+      es_fprintf (fp, ", ot=%d, d=%d, vl=%lu\n", rec->r.trust.ownertrust,
+                  rec->r.trust.depth, rec->r.trust.validlist);
       break;
 
     case RECTYPE_VALID:
       es_fprintf (fp, "valid ");
       for (i=0; i < 20; i++)
         es_fprintf(fp, "%02X", rec->r.valid.namehash[i]);
-      es_fprintf (fp, ", v=%d, next=%lu, f=%d, m=%d\n",
-                  rec->r.valid.validity, rec->r.valid.next,
-                  rec->r.valid.full_count, rec->r.valid.marginal_count);
+      es_fprintf (fp, ", v=%d, next=%lu\n", rec->r.valid.validity,
+                  rec->r.valid.next);
       break;
 
     default:
@@ -1604,7 +1629,7 @@ tdbio_read_record (ulong recnum, TRUSTREC *rec, int expected)
       rec->r.trust.ownertrust = *p++;
       rec->r.trust.depth = *p++;
       rec->r.trust.min_ownertrust = *p++;
-      rec->r.trust.flags = *p++;
+      p++;
       rec->r.trust.validlist = buf32_to_ulong(p);
       break;
 
@@ -1699,7 +1724,7 @@ tdbio_write_record (ctrl_t ctrl, TRUSTREC *rec)
       *p++ = rec->r.trust.ownertrust;
       *p++ = rec->r.trust.depth;
       *p++ = rec->r.trust.min_ownertrust;
-      *p++ = rec->r.trust.flags;
+      p++;
       ulongtobuf( p, rec->r.trust.validlist); p += 4;
       break;
 
@@ -1864,21 +1889,13 @@ cmp_trec_fpr ( const void *fpr, const TRUSTREC *rec )
  * Return: 0 if found, GPG_ERR_NOT_FOUND, or another error code.
  */
 gpg_error_t
-tdbio_search_trust_byfpr (ctrl_t ctrl, const byte *fpr, unsigned int fprlen,
-                          TRUSTREC *rec)
+tdbio_search_trust_byfpr (ctrl_t ctrl, const byte *fingerprint, TRUSTREC *rec)
 {
   int rc;
-  byte fingerprint[20];
-
-  if (fprlen != 20)
-    {
-      fpr20_from_fpr (fpr, fprlen, fingerprint);
-      fpr = fingerprint;
-    }
 
   /* Locate the trust record using the hash table */
-  rc = lookup_hashtable (get_trusthashrec (ctrl), fpr, 20,
-                         cmp_trec_fpr, fpr, rec);
+  rc = lookup_hashtable (get_trusthashrec (ctrl), fingerprint, 20,
+                         cmp_trec_fpr, fingerprint, rec );
   return rc;
 }
 
@@ -1892,10 +1909,13 @@ tdbio_search_trust_byfpr (ctrl_t ctrl, const byte *fpr, unsigned int fprlen,
 gpg_error_t
 tdbio_search_trust_bypk (ctrl_t ctrl, PKT_public_key *pk, TRUSTREC *rec)
 {
-  byte fingerprint[20];
+  byte fingerprint[MAX_FINGERPRINT_LEN];
+  size_t fingerlen;
 
-  fpr20_from_pk (pk, fingerprint);
-  return tdbio_search_trust_byfpr (ctrl, fingerprint, 20, rec);
+  fingerprint_from_pk( pk, fingerprint, &fingerlen );
+  for (; fingerlen < 20; fingerlen++)
+    fingerprint[fingerlen] = 0;
+  return tdbio_search_trust_byfpr (ctrl, fingerprint, rec);
 }
 
 
