@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef G13_COMMON_H
@@ -32,7 +32,6 @@
 #include "../common/session-env.h"
 #include "../common/strlist.h"
 
-
 /* Debug values and macros.  */
 #define DBG_MOUNT_VALUE     1	/* Debug mount or device stuff. */
 #define DBG_CRYPTO_VALUE    4	/* Debug low level crypto.  */
@@ -48,6 +47,7 @@
 /* A large struct named "opt" to keep global flags.  Note that this
    struct is used by g13 and g13-syshelp and thus some fields may only
    make sense for one of them.  */
+EXTERN_UNLESS_MAIN_MODULE
 struct
 {
   unsigned int debug; /* Debug flags (DBG_foo_VALUE).  */
@@ -55,14 +55,13 @@ struct
   int quiet;          /* Be as quiet as possible.  */
   int dry_run;        /* Don't change any persistent data.  */
 
-  const char *homedir;         /* Configuration directory name.  */
   const char *config_filename; /* Name of the used config file.  */
 
   /* Filename of the AGENT program.  */
   const char *agent_program;
 
   /* Filename of the GPG program.  Unless set via an program option it
-     is initialzed at the first engine startup to the standard gpg
+     is initialized at the first engine startup to the standard gpg
      filename.  */
   const char *gpg_program;
 
