@@ -24,6 +24,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <time.h>
+#include <assert.h>
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
@@ -37,7 +38,7 @@
 
 #include "keydb.h"
 #include "../common/i18n.h"
-#include "../common/membuf.h"
+
 
 struct dn_array_s {
   char *key;
@@ -47,28 +48,6 @@ struct dn_array_s {
 };
 
 
-/* Get the first first element from the s-expression SN and return a
- * pointer to it.  Stores the length at R_LENGTH.  Returns NULL for no
- * value or an invalid expression.  */
-const void *
-gpgsm_get_serial (ksba_const_sexp_t sn, size_t *r_length)
-{
-  const char *p = (const char *)sn;
-  unsigned long n;
-  char *endp;
-
-  if (!p || *p != '(')
-    return NULL;
-  p++;
-  n = strtoul (p, &endp, 10);
-  p = endp;
-  if (*p++ != ':')
-    return NULL;
-  *r_length = n;
-  return p;
-}
-
-
 /* Print the first element of an S-Expression. */
 void
 gpgsm_print_serial (estream_t fp, ksba_const_sexp_t sn)
@@ -94,74 +73,6 @@ gpgsm_print_serial (estream_t fp, ksba_const_sexp_t sn)
 }
 
 
-/* Print the first element of an S-Expression in decimal notation
- * assuming it is a non-negative integer. */
-void
-gpgsm_print_serial_decimal (estream_t fp, ksba_const_sexp_t sn)
-{
-  const char *p = (const char *)sn;
-  unsigned long n, i;
-  char *endp;
-  gcry_mpi_t a, r, ten;
-  unsigned int dd;
-
-  if (!p)
-    es_fputs (_("none"), fp);
-  else if (*p != '(')
-    es_fputs ("[Internal error - not an S-expression]", fp);
-  else
-    {
-      p++;
-      n = strtoul (p, &endp, 10);
-      p = endp;
-      if (*p++ != ':')
-        es_fputs ("[Internal Error - invalid S-expression]", fp);
-      else if (gcry_mpi_scan (&a, GCRYMPI_FMT_USG, p, n, NULL))
-        es_fputs ("[Internal Error - can't convert to decimal]", fp);
-      else
-        {
-          membuf_t mb = MEMBUF_ZERO;
-          char *buf;
-          int c;
-
-          ten = gcry_mpi_set_ui (NULL, 10);
-          r = gcry_mpi_new (0);
-
-          do
-            {
-              gcry_mpi_div (a, r, a, ten, 0);
-              gcry_mpi_get_ui (&dd, r);
-              put_membuf_printf (&mb, "%u", dd);
-            }
-          while (gcry_mpi_cmp_ui (a, 0));
-
-          /* Make sure we have at least an empty string, get it,
-           * reverse it, and print it. */
-          put_membuf (&mb, "", 1);
-          buf = get_membuf (&mb, NULL);
-          if (!buf)
-            es_fputs ("[Internal Error - out of core]", fp);
-          else
-            {
-              n = strlen (buf);
-              for (i=0; i < n/2; i++)
-                {
-                  c = buf[i];
-                  buf[i] = buf[n-1-i];
-                  buf[n-1-i] = c;
-                }
-              es_fputs (buf, fp);
-              xfree (buf);
-            }
-
-          gcry_mpi_release (r);
-          gcry_mpi_release (ten);
-          gcry_mpi_release (a);
-        }
-    }
-}
-
-
 /* Dump the serial number or any other simple S-expression. */
 void
 gpgsm_dump_serial (ksba_const_sexp_t sn)
@@ -256,7 +167,7 @@ gpgsm_dump_string (const char *string)
       else
         {
           log_printf ( "[ ");
-          log_printhex (string, strlen (string), NULL);
+          log_printhex (NULL, string, strlen (string));
           log_printf ( " ]");
         }
     }
@@ -385,7 +296,7 @@ parse_dn_part (struct dn_array_s *array, const unsigned char *string)
     const char *oid;
   } label_map[] = {
     /* Warning: When adding new labels, make sure that the buffer
-       array->key will be allocated large enough. */
+       below we be allocated large enough. */
     {"EMail",        "1.2.840.113549.1.9.1" },
     {"T",            "2.5.4.12" },
     {"GN",           "2.5.4.42" },
@@ -397,7 +308,6 @@ parse_dn_part (struct dn_array_s *array, const unsigned char *string)
     {"PostalCode",   "2.5.4.17" },
     {"Pseudo",       "2.5.4.65" },
     {"SerialNumber", "2.5.4.5" },
-    {"Callsign",     "1.3.6.1.4.1.12348.1.1"},
     {NULL, NULL}
   };
   const unsigned char *s, *s1;
@@ -438,7 +348,7 @@ parse_dn_part (struct dn_array_s *array, const unsigned char *string)
     { /* hexstring */
       string++;
       for (s=string; hexdigitp (s); s++)
-        ;
+        s++;
       n = s - string;
       if (!n || (n & 1))
         return NULL; /* Empty or odd number of digits. */
@@ -643,7 +553,7 @@ pretty_es_print_sexp (estream_t fp, const unsigned char *buf, size_t buflen)
       return;
     }
   len = gcry_sexp_sprint (sexp, GCRYSEXP_FMT_ADVANCED, NULL, 0);
-  log_assert (len);
+  assert (len);
   result = xtrymalloc (len);
   if (!result)
     {
@@ -652,7 +562,7 @@ pretty_es_print_sexp (estream_t fp, const unsigned char *buf, size_t buflen)
       return;
     }
   len = gcry_sexp_sprint (sexp, GCRYSEXP_FMT_ADVANCED, result, len);
-  log_assert (len);
+  assert (len);
   for (p = result; len; len--, p++)
     {
       if (*p == '\n')
@@ -728,14 +638,7 @@ gpgsm_es_print_name2 (estream_t fp, const char *name, int translate)
 void
 gpgsm_es_print_name (estream_t fp, const char *name)
 {
-  if (opt.no_pretty_dn)
-    {
-      if (!name)
-        name = "[error]";
-      es_write_sanitized (fp, name, strlen (name), NULL, NULL);
-    }
-  else
-    gpgsm_es_print_name2 (fp, name, 1);
+  gpgsm_es_print_name2 (fp, name, 1);
 }
 
 
@@ -755,7 +658,6 @@ format_name_writer (void *cookie, const void *buffer, size_t size)
   struct format_name_cookie *c = cookie;
   char *p;
 
-  /* FIXME: Replace the whole thing using es_fopenmem based code.  */
   if (!buffer)  /* Flush. */
     return 0;  /* (Actually we could use SIZE because that should be 0 too.)  */
 
