@@ -59,6 +59,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <time.h>
+#include <assert.h>
 
 #include "gpgsm.h"
 #include <gcrypt.h>
@@ -66,7 +67,6 @@
 
 #include "keydb.h"
 #include "../common/i18n.h"
-#include "../common/membuf.h"
 
 
 enum para_name
@@ -74,7 +74,6 @@ enum para_name
     pKEYTYPE,
     pKEYLENGTH,
     pKEYGRIP,
-    pKEYCURVE,
     pKEYUSAGE,
     pNAMEDN,
     pNAMEEMAIL,
@@ -199,7 +198,7 @@ parse_parameter_usage (struct para_data_s *para, enum para_name key)
         use |= GCRY_PK_USAGE_CERT;
       else
         {
-          log_error ("line %d: invalid usage list\n", r?r->lnr:0);
+          log_error ("line %d: invalid usage list\n", r->lnr);
           return -1; /* error */
         }
     }
@@ -237,7 +236,6 @@ read_parameters (ctrl_t ctrl, estream_t fp, estream_t out_fp)
     { "Key-Type",       pKEYTYPE},
     { "Key-Length",     pKEYLENGTH },
     { "Key-Grip",       pKEYGRIP },
-    { "Key-Curve",      pKEYCURVE },
     { "Key-Usage",      pKEYUSAGE },
     { "Name-DN",        pNAMEDN },
     { "Name-Email",     pNAMEEMAIL, 1 },
@@ -435,7 +433,6 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
   struct para_data_s *r;
   const char *s, *string;
   int i;
-  int algo;
   unsigned int nbits;
   char numbuf[20];
   unsigned char keyparms[100];
@@ -447,27 +444,24 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
   char *cardkeyid = NULL;
 
   /* Check that we have all required parameters; */
-  log_assert (get_parameter (para, pKEYTYPE, 0));
+  assert (get_parameter (para, pKEYTYPE, 0));
 
-  /* There is a problem with pkcs-10 on how to use ElGamal because it
-     is expected that a PK algorithm can always be used for
-     signing.  Another problem is that on-card generated encryption
-     keys may not be used for signing.  */
-  algo = get_parameter_algo (para, pKEYTYPE);
-  if (!algo && (s = get_parameter_value (para, pKEYTYPE, 0)) && *s)
+  /* We can only use RSA for now.  There is a problem with pkcs-10 on
+     how to use ElGamal because it is expected that a PK algorithm can
+     always be used for signing. Another problem is that on-card
+     generated encryption keys may not be used for signing.  */
+  i = get_parameter_algo (para, pKEYTYPE);
+  if (!i && (s = get_parameter_value (para, pKEYTYPE, 0)) && *s)
     {
       /* Hack to allow creation of certificates directly from a smart
          card.  For example: "Key-Type: card:OPENPGP.3".  */
       if (!strncmp (s, "card:", 5) && s[5])
         cardkeyid = xtrystrdup (s+5);
     }
-  if (algo < 1 && !cardkeyid)
+  if ( (i < 1 || i != GCRY_PK_RSA) && !cardkeyid )
     {
       r = get_parameter (para, pKEYTYPE, 0);
-      if (r)
-        log_error (_("line %d: invalid algorithm\n"), r?r->lnr:0);
-      else
-        log_error ("No Key-Type specified\n");
+      log_error (_("line %d: invalid algorithm\n"), r->lnr);
       return gpg_error (GPG_ERR_INV_PARAMETER);
     }
 
@@ -482,7 +476,7 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
       /* The BSI specs dated 2002-11-25 don't allow lengths below 1024. */
       r = get_parameter (para, pKEYLENGTH, 0);
       log_error (_("line %d: invalid key length %u (valid are %d to %d)\n"),
-                 r?r->lnr:0, nbits, 1024, 4096);
+                 r->lnr, nbits, 1024, 4096);
       xfree (cardkeyid);
       return gpg_error (GPG_ERR_INV_PARAMETER);
     }
@@ -499,7 +493,7 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
   if (!(s=get_parameter_value (para, pNAMEDN, 0)))
     {
       r = get_parameter (para, pNAMEDN, 0);
-      log_error (_("line %d: no subject name given\n"), r?r->lnr:0);
+      log_error (_("line %d: no subject name given\n"), r->lnr);
       xfree (cardkeyid);
       return gpg_error (GPG_ERR_INV_PARAMETER);
     }
@@ -509,10 +503,10 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
       r = get_parameter (para, pNAMEDN, 0);
       if (gpg_err_code (err) == GPG_ERR_UNKNOWN_NAME)
         log_error (_("line %d: invalid subject name label '%.*s'\n"),
-                   r?r->lnr:0, (int)errlen, s+erroff);
+                   r->lnr, (int)errlen, s+erroff);
       else
         log_error (_("line %d: invalid subject name '%s' at pos %d\n"),
-                   r?r->lnr:0, s, (int)erroff);
+                   r->lnr, s, (int)erroff);
 
       xfree (cardkeyid);
       return gpg_error (GPG_ERR_INV_PARAMETER);
@@ -528,7 +522,7 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
           || strstr(s, ".."))
         {
           r = get_parameter (para, pNAMEEMAIL, seq);
-          log_error (_("line %d: not a valid email address\n"), r?r->lnr:0);
+          log_error (_("line %d: not a valid email address\n"), r->lnr);
           xfree (cardkeyid);
           return gpg_error (GPG_ERR_INV_PARAMETER);
         }
@@ -547,7 +541,7 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
           if (*s)
             {
               r = get_parameter (para, pSERIAL, 0);
-              log_error (_("line %d: invalid serial number\n"), r?r->lnr:0);
+              log_error (_("line %d: invalid serial number\n"), r->lnr);
               xfree (cardkeyid);
               return gpg_error (GPG_ERR_INV_PARAMETER);
             }
@@ -564,10 +558,10 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
           r = get_parameter (para, pISSUERDN, 0);
           if (gpg_err_code (err) == GPG_ERR_UNKNOWN_NAME)
             log_error (_("line %d: invalid issuer name label '%.*s'\n"),
-                       r?r->lnr:0, (int)errlen, string+erroff);
+                       r->lnr, (int)errlen, string+erroff);
           else
             log_error (_("line %d: invalid issuer name '%s' at pos %d\n"),
-                       r?r->lnr:0, string, (int)erroff);
+                       r->lnr, string, (int)erroff);
           xfree (cardkeyid);
           return gpg_error (GPG_ERR_INV_PARAMETER);
         }
@@ -578,7 +572,7 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
   if (string && !string2isotime (NULL, string))
     {
       r = get_parameter (para, pNOTBEFORE, 0);
-      log_error (_("line %d: invalid date given\n"), r?r->lnr:0);
+      log_error (_("line %d: invalid date given\n"), r->lnr);
       xfree (cardkeyid);
       return gpg_error (GPG_ERR_INV_PARAMETER);
     }
@@ -589,7 +583,7 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
   if (string && !string2isotime (NULL, string))
     {
       r = get_parameter (para, pNOTAFTER, 0);
-      log_error (_("line %d: invalid date given\n"), r?r->lnr:0);
+      log_error (_("line %d: invalid date given\n"), r->lnr);
       xfree (cardkeyid);
       return gpg_error (GPG_ERR_INV_PARAMETER);
     }
@@ -603,7 +597,7 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
         {
           r = get_parameter (para, pKEYTYPE, 0);
           log_error (_("line %d: error getting signing key by keygrip '%s'"
-                       ": %s\n"), r?r->lnr:0, s, gpg_strerror (rc));
+                       ": %s\n"), r->lnr, s, gpg_strerror (rc));
           xfree (cardkeyid);
           return rc;
         }
@@ -621,7 +615,7 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
                         || mdalgo == GCRY_MD_SHA512)))
       {
         r = get_parameter (para, pHASHALGO, 0);
-        log_error (_("line %d: invalid hash algorithm given\n"), r?r->lnr:0);
+        log_error (_("line %d: invalid hash algorithm given\n"), r->lnr);
         xfree (cardkeyid);
         return gpg_error (GPG_ERR_INV_PARAMETER);
       }
@@ -629,14 +623,14 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
 
   /* Check the optional AuthorityKeyId.  */
   string = get_parameter_value (para, pAUTHKEYID, 0);
-  if (string && strcmp (string, "none"))
+  if (string)
     {
       for (s=string, i=0; hexdigitp (s); s++, i++)
         ;
       if (*s || (i&1))
         {
           r = get_parameter (para, pAUTHKEYID, 0);
-          log_error (_("line %d: invalid authority-key-id\n"), r?r->lnr:0);
+          log_error (_("line %d: invalid authority-key-id\n"), r->lnr);
           xfree (cardkeyid);
           return gpg_error (GPG_ERR_INV_PARAMETER);
         }
@@ -644,14 +638,14 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
 
   /* Check the optional SubjectKeyId.  */
   string = get_parameter_value (para, pSUBJKEYID, 0);
-  if (string && strcmp (string, "none"))
+  if (string)
     {
       for (s=string, i=0; hexdigitp (s); s++, i++)
         ;
       if (*s || (i&1))
         {
           r = get_parameter (para, pSUBJKEYID, 0);
-          log_error (_("line %d: invalid subject-key-id\n"), r?r->lnr:0);
+          log_error (_("line %d: invalid subject-key-id\n"), r->lnr);
           xfree (cardkeyid);
           return gpg_error (GPG_ERR_INV_PARAMETER);
         }
@@ -689,7 +683,7 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
       if (!okay)
         {
           r = get_parameter (para, pEXTENSION, seq);
-          log_error (_("line %d: invalid extension syntax\n"), r? r->lnr:0);
+          log_error (_("line %d: invalid extension syntax\n"), r->lnr);
           xfree (cardkeyid);
           return gpg_error (GPG_ERR_INV_PARAMETER);
         }
@@ -703,7 +697,7 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
         {
           r = get_parameter (para, pKEYTYPE, 0);
           log_error (_("line %d: error reading key '%s' from card: %s\n"),
-                     r?r->lnr:0, cardkeyid, gpg_strerror (rc));
+                     r->lnr, cardkeyid, gpg_strerror (rc));
           xfree (sigkey);
           xfree (cardkeyid);
           return rc;
@@ -724,43 +718,16 @@ proc_parameters (ctrl_t ctrl, struct para_data_s *para,
     }
   else if (!outctrl->dryrun) /* Generate new key.  */
     {
-      if (algo == GCRY_PK_RSA)
-        {
-          sprintf (numbuf, "%u", nbits);
-          snprintf ((char*)keyparms, DIM (keyparms),
-                    "(6:genkey(3:rsa(5:nbits%d:%s)))",
-                    (int)strlen (numbuf), numbuf);
-        }
-      else if (algo == GCRY_PK_ECC)
-        {
-          const char *curve = get_parameter_value (para, pKEYCURVE, 0);
-          const char *flags;
-
-          if (!strcmp (curve, "Ed25519"))
-            flags = "(5:flags5:eddsa)";
-          else if (!strcmp (curve, "Curve25519"))
-            flags = "(5:flags9:djb-tweak)";
-          else
-            flags = "";
-
-          snprintf ((char*)keyparms, DIM (keyparms),
-                    "(6:genkey(3:ecc(5:curve%zu:%s)%s))",
-                    strlen (curve), curve, flags);
-        }
-      else
-        {
-          r = get_parameter (para, pKEYTYPE, 0);
-          log_error (_("line %d: invalid algorithm\n"), r?r->lnr:0);
-          xfree (sigkey);
-          xfree (cardkeyid);
-          return gpg_error (GPG_ERR_INV_PARAMETER);
-        }
+      sprintf (numbuf, "%u", nbits);
+      snprintf ((char*)keyparms, DIM (keyparms),
+                "(6:genkey(3:rsa(5:nbits%d:%s)))",
+                (int)strlen (numbuf), numbuf);
       rc = gpgsm_agent_genkey (ctrl, keyparms, &public);
       if (rc)
         {
           r = get_parameter (para, pKEYTYPE, 0);
           log_error (_("line %d: key generation failed: %s <%s>\n"),
-                     r?r->lnr:0, gpg_strerror (rc), gpg_strsource (rc));
+                     r->lnr, gpg_strerror (rc), gpg_strsource (rc));
           xfree (sigkey);
           xfree (cardkeyid);
           return rc;
@@ -835,49 +802,26 @@ create_request (ctrl_t ctrl,
   ksba_isotime_t atime;
   int certmode = 0;
   int mdalgo;
-  membuf_t tbsbuffer;
-  membuf_t *tbsmb = NULL;
-  size_t publiclen;
-  size_t sigkeylen;
-  int publicpkalgo;  /* The gcrypt public key algo of the public key.  */
-  int sigkeypkalgo;  /* The gcrypt public key algo of the signing key.  */
 
   err = ksba_certreq_new (&cr);
   if (err)
     return err;
 
-  publiclen = gcry_sexp_canon_len (public, 0, NULL, NULL);
-  sigkeylen = sigkey? gcry_sexp_canon_len (sigkey, 0, NULL, NULL) : 0;
-
-  publicpkalgo = get_pk_algo_from_canon_sexp (public, publiclen);
-  sigkeypkalgo = sigkey? get_pk_algo_from_canon_sexp (public, publiclen) : 0;
-
-  if (publicpkalgo == GCRY_PK_EDDSA)
-    {
-      mdalgo = GCRY_MD_SHA512;
-      md = NULL;  /* We sign the data and not a hash.  */
-      init_membuf (&tbsbuffer, 2048);
-      tbsmb = &tbsbuffer;
-      ksba_certreq_set_hash_function
-        (cr, (void (*)(void *, const void*,size_t))put_membuf, tbsmb);
-    }
+  string = get_parameter_value (para, pHASHALGO, 0);
+  if (string)
+    mdalgo = gcry_md_map_name (string);
   else
+    mdalgo = GCRY_MD_SHA256;
+  rc = gcry_md_open (&md, mdalgo, 0);
+  if (rc)
     {
-      if ((string = get_parameter_value (para, pHASHALGO, 0)))
-        mdalgo = gcry_md_map_name (string);
-      else
-        mdalgo = GCRY_MD_SHA256;
-      rc = gcry_md_open (&md, mdalgo, 0);
-      if (rc)
-        {
-          log_error ("md_open failed: %s\n", gpg_strerror (rc));
-          goto leave;
-        }
-      if (DBG_HASHING)
-        gcry_md_debug (md, "cr.cri");
-      ksba_certreq_set_hash_function (cr, HASH_FNC, md);
+      log_error ("md_open failed: %s\n", gpg_strerror (rc));
+      goto leave;
     }
+  if (DBG_HASHING)
+    gcry_md_debug (md, "cr.cri");
 
+  ksba_certreq_set_hash_function (cr, HASH_FNC, md);
   ksba_certreq_set_writer (cr, writer);
 
   err = ksba_certreq_add_subject (cr, get_parameter_value (para, pNAMEDN, 0));
@@ -914,7 +858,7 @@ create_request (ctrl_t ctrl,
   for (seq=0; (s = get_parameter_value (para, pNAMEDNS, seq)); seq++)
     {
       len = strlen (s);
-      log_assert (len);
+      assert (len);
       snprintf (numbuf, DIM(numbuf), "%u:", (unsigned int)len);
       buf = p = xtrymalloc (11 + strlen (numbuf) + len + 3);
       if (!buf)
@@ -941,7 +885,7 @@ create_request (ctrl_t ctrl,
   for (seq=0; (s = get_parameter_value (para, pNAMEURI, seq)); seq++)
     {
       len = strlen (s);
-      log_assert (len);
+      assert (len);
       snprintf (numbuf, DIM(numbuf), "%u:", (unsigned int)len);
       buf = p = xtrymalloc (6 + strlen (numbuf) + len + 3);
       if (!buf)
@@ -965,10 +909,12 @@ create_request (ctrl_t ctrl,
         }
     }
 
+
   err = ksba_certreq_set_public_key (cr, public);
   if (err)
     {
-      log_error ("error setting the public key: %s\n", gpg_strerror (err));
+      log_error ("error setting the public key: %s\n",
+                 gpg_strerror (err));
       rc = err;
       goto leave;
     }
@@ -1164,17 +1110,14 @@ create_request (ctrl_t ctrl,
          given we set it to the public key to create a self-signed
          certificate. */
       if (!sigkey)
-        {
-          sigkey = public;
-          sigkeylen = publiclen;
-          sigkeypkalgo = publicpkalgo;
-        }
+        sigkey = public;
 
-      /* Set the the digestinfo aka siginfo.  */
       {
         unsigned char *siginfo;
 
-        err = transform_sigval (sigkey, sigkeylen, mdalgo, &siginfo, NULL);
+        err = transform_sigval (sigkey,
+                                gcry_sexp_canon_len (sigkey, 0, NULL, NULL),
+                                mdalgo, &siginfo, NULL);
         if (!err)
           {
             err = ksba_certreq_set_siginfo (cr, siginfo);
@@ -1189,12 +1132,9 @@ create_request (ctrl_t ctrl,
           }
       }
 
-
       /* Insert the AuthorityKeyId.  */
       string = get_parameter_value (para, pAUTHKEYID, 0);
-      if (string && !strcmp (string, "none"))
-        ; /* Do not issue an AKI.  */
-      else if (string)
+      if (string)
         {
           char *hexbuf;
 
@@ -1220,69 +1160,20 @@ create_request (ctrl_t ctrl,
           hexbuf[2] = 0x80;  /* Context tag for an implicit Octet string.  */
           hexbuf[3] = len;
           err = ksba_certreq_add_extension (cr, oidstr_authorityKeyIdentifier,
-                                            0, hexbuf, 4+len);
+                                            0,
+                                            hexbuf, 4+len);
           xfree (hexbuf);
           if (err)
             {
-              log_error ("error setting the AKI: %s\n", gpg_strerror (err));
-              goto leave;
-            }
-        }
-      else if (publicpkalgo == GCRY_PK_EDDSA || publicpkalgo == GCRY_PK_ECC)
-        {
-          /* For EdDSA and ECC we add the public key as default identifier.  */
-          const unsigned char *q;
-          size_t qlen, derlen;
-          unsigned char *der;
-
-          err = get_ecc_q_from_canon_sexp (public, publiclen, &q, &qlen);
-          if (err)
-            {
-              log_error ("error getting Q from public key: %s\n",
+              log_error ("error setting the authority-key-id: %s\n",
                          gpg_strerror (err));
               goto leave;
             }
-          if (publicpkalgo == GCRY_PK_EDDSA && qlen>32 && (qlen&1) && *q==0x40)
-            {
-              /* Skip our optional native encoding octet.  */
-              q++;
-              qlen--;
-            }
-          /* FIXME: For plain ECC we should better use a compressed
-           * point.  That requires an updated Libgcrypt.  Without that
-           * using nistp521 won't work due to the length check below.  */
-          if (qlen > 125)
-            {
-              err = gpg_error (GPG_ERR_TOO_LARGE);
-              goto leave;
-            }
-          derlen = 4 + qlen;
-          der = xtrymalloc (derlen);
-          if (!der)
-            {
-              err = gpg_error_from_syserror ();
-              goto leave;
-            }
-          der[0] = 0x30; /* Sequence */
-          der[1] = qlen + 2;
-          der[2] = 0x80; /* Context tag for an implict Octet String. */
-          der[3] = qlen;
-          memcpy (der+4, q, qlen);
-          err = ksba_certreq_add_extension (cr, oidstr_authorityKeyIdentifier,
-                                            0, der, derlen);
-          xfree (der);
-          if (err)
-            {
-              log_error ("error setting the AKI: %s\n", gpg_strerror (err));
-              goto leave;
-            }
         }
 
       /* Insert the SubjectKeyId.  */
       string = get_parameter_value (para, pSUBJKEYID, 0);
-      if (string && !strcmp (string, "none"))
-        ; /* Do not issue an SKI.  */
-      else if (string)
+      if (string)
         {
           char *hexbuf;
 
@@ -1310,57 +1201,10 @@ create_request (ctrl_t ctrl,
           xfree (hexbuf);
           if (err)
             {
-              log_error ("error setting SKI: %s\n", gpg_strerror (err));
-              goto leave;
-            }
-        }
-      else if (sigkeypkalgo == GCRY_PK_EDDSA || sigkeypkalgo == GCRY_PK_ECC)
-        {
-          /* For EdDSA and ECC we add the public key as default identifier.  */
-          const unsigned char *q;
-          size_t qlen, derlen;
-          unsigned char *der;
-
-          /* FIXME: This assumes that the to-be-certified key uses the
-           * same algorithm as the certification key - this is not
-           * always the case; in fact it is common that they
-           * differ.  */
-          err = get_ecc_q_from_canon_sexp (sigkey, sigkeylen, &q, &qlen);
-          if (err)
-            {
-              log_error ("error getting Q from signature key: %s\n",
+              log_error ("error setting the subject-key-id: %s\n",
                          gpg_strerror (err));
               goto leave;
             }
-          if (sigkeypkalgo == GCRY_PK_EDDSA && qlen>32 && (qlen&1) && *q==0x40)
-            {
-              /* Skip our optional native encoding octet.  */
-              q++;
-              qlen--;
-            }
-          if (qlen > 127)
-            {
-              err = gpg_error (GPG_ERR_TOO_LARGE);
-              goto leave;
-            }
-          derlen = 2 + qlen;
-          der = xtrymalloc (derlen);
-          if (!der)
-            {
-              err = gpg_error_from_syserror ();
-              goto leave;
-            }
-          der[0] = 0x04; /* Octet String */
-          der[1] = qlen;
-          memcpy (der+2, q, qlen);
-          err = ksba_certreq_add_extension (cr, oidstr_subjectKeyIdentifier, 0,
-                                            der, derlen);
-          xfree (der);
-          if (err)
-            {
-              log_error ("error setting the SKI: %s\n", gpg_strerror (err));
-              goto leave;
-            }
         }
 
       /* Insert additional extensions.  */
@@ -1422,11 +1266,7 @@ create_request (ctrl_t ctrl,
         }
     }
   else
-    {
-      sigkey = public;
-      sigkeylen = publiclen;
-      sigkeypkalgo = publicpkalgo;
-    }
+    sigkey = public;
 
   do
     {
@@ -1440,14 +1280,20 @@ create_request (ctrl_t ctrl,
       if (stopreason == KSBA_SR_NEED_SIG)
         {
           gcry_sexp_t s_pkey;
+          size_t n;
           unsigned char grip[20];
           char hexgrip[41];
           unsigned char *sigval, *newsigval;
           size_t siglen;
-          void *tbsdata;
-          size_t tbsdatalen;
 
-          rc = gcry_sexp_sscan (&s_pkey, NULL, (const char*)sigkey, sigkeylen);
+          n = gcry_sexp_canon_len (sigkey, 0, NULL, NULL);
+          if (!n)
+            {
+              log_error ("libksba did not return a proper S-Exp\n");
+              rc = gpg_error (GPG_ERR_BUG);
+              goto leave;
+            }
+          rc = gcry_sexp_sscan (&s_pkey, NULL, (const char*)sigkey, n);
           if (rc)
             {
               log_error ("gcry_sexp_scan failed: %s\n", gpg_strerror (rc));
@@ -1463,31 +1309,15 @@ create_request (ctrl_t ctrl,
           gcry_sexp_release (s_pkey);
           bin2hex (grip, 20, hexgrip);
 
-          if (!opt.quiet)
-            log_info ("about to sign the %s for key: &%s\n",
-                      certmode? "certificate":"CSR", hexgrip);
+          log_info ("about to sign the %s for key: &%s\n",
+                    certmode? "certificate":"CSR", hexgrip);
 
           if (carddirect && !certmode)
-            {
-              if (tbsmb)
-                {
-                  tbsdata = get_membuf (tbsmb, &tbsdatalen);
-                  tbsmb = NULL;
-                  if (!tbsdata)
-                    rc = gpg_error_from_syserror ();
-                  else
-                    rc = gpgsm_scd_pksign (ctrl, carddirect, NULL,
-                                           tbsdata, tbsdatalen, 0,
-                                           &sigval, &siglen);
-                  xfree (tbsdata);
-                }
-              else
-                rc = gpgsm_scd_pksign (ctrl, carddirect, NULL,
-                                       gcry_md_read (md, mdalgo),
-                                       gcry_md_get_algo_dlen (mdalgo),
-                                       mdalgo,
-                                       &sigval, &siglen);
-            }
+            rc = gpgsm_scd_pksign (ctrl, carddirect, NULL,
+                                   gcry_md_read (md, mdalgo),
+                                   gcry_md_get_algo_dlen (mdalgo),
+                                   mdalgo,
+                                   &sigval, &siglen);
           else
             {
               char *orig_codeset;
@@ -1499,25 +1329,11 @@ create_request (ctrl_t ctrl,
                    " the passphrase for the key you just created once"
                    " more.\n"));
               i18n_switchback (orig_codeset);
-              if (tbsmb)
-                {
-                  tbsdata = get_membuf (tbsmb, &tbsdatalen);
-                  tbsmb = NULL;
-                  if (!tbsdata)
-                    rc = gpg_error_from_syserror ();
-                  else
-                    rc = gpgsm_agent_pksign (ctrl, hexgrip, desc,
-                                             tbsdata, tbsdatalen, 0,
-
-                                             &sigval, &siglen);
-                  xfree (tbsdata);
-                }
-              else
-                rc = gpgsm_agent_pksign (ctrl, hexgrip, desc,
-                                         gcry_md_read(md, mdalgo),
-                                         gcry_md_get_algo_dlen (mdalgo),
-                                         mdalgo,
-                                         &sigval, &siglen);
+              rc = gpgsm_agent_pksign (ctrl, hexgrip, desc,
+                                       gcry_md_read(md, mdalgo),
+                                       gcry_md_get_algo_dlen (mdalgo),
+                                       mdalgo,
+                                       &sigval, &siglen);
               xfree (desc);
             }
           if (rc)
@@ -1526,7 +1342,8 @@ create_request (ctrl_t ctrl,
               goto leave;
             }
 
-          err = transform_sigval (sigval, siglen, mdalgo, &newsigval, NULL);
+          err = transform_sigval (sigval, siglen, mdalgo,
+                                  &newsigval, NULL);
           xfree (sigval);
           if (!err)
             {
@@ -1546,8 +1363,6 @@ create_request (ctrl_t ctrl,
 
 
  leave:
-  if (tbsmb)
-    xfree (get_membuf (tbsmb, NULL));
   gcry_md_close (md);
   ksba_certreq_release (cr);
   return rc;
