@@ -75,10 +75,33 @@ static void unlock_all (KEYDB_HANDLE hd);
 static void
 try_make_homedir (const char *fname)
 {
+  const char *defhome = standard_homedir ();
+
+  /* Create the directory only if the supplied directory name is the
+     same as the default one.  This way we avoid to create arbitrary
+     directories when a non-default home directory is used.  To cope
+     with HOME, we do compare only the suffix if we see that the
+     default homedir does start with a tilde.  */
   if ( opt.dry_run || opt.no_homedir_creation )
     return;
 
-  gnupg_maybe_make_homedir (fname, opt.quiet);
+  if (
+#ifdef HAVE_W32_SYSTEM
+      ( !compare_filenames (fname, defhome) )
+#else
+      ( *defhome == '~'
+        && (strlen(fname) >= strlen (defhome+1)
+            && !strcmp(fname+strlen(fname)-strlen(defhome+1), defhome+1 ) ))
+      || (*defhome != '~'  && !compare_filenames( fname, defhome ) )
+#endif
+      )
+    {
+      if (gnupg_mkdir (fname, "-rwx"))
+        log_info (_("can't create directory '%s': %s\n"),
+                  fname, strerror(errno) );
+      else if (!opt.quiet )
+        log_info (_("directory '%s' created\n"), fname);
+    }
 }
 
 
@@ -90,9 +113,8 @@ try_make_homedir (const char *fname)
 static gpg_error_t
 maybe_create_keybox (char *filename, int force, int *r_created)
 {
-  gpg_err_code_t ec;
   dotlock_t lockhd = NULL;
-  estream_t fp;
+  FILE *fp;
   int rc;
   mode_t oldmask;
   char *last_slash_in_filename;
@@ -102,8 +124,8 @@ maybe_create_keybox (char *filename, int force, int *r_created)
     *r_created = 0;
 
   /* A quick test whether the filename already exists. */
-  if (!gnupg_access (filename, F_OK))
-    return !gnupg_access (filename, R_OK)? 0 : gpg_error (GPG_ERR_EACCES);
+  if (!access (filename, F_OK))
+    return !access (filename, R_OK)? 0 : gpg_error (GPG_ERR_EACCES);
 
   /* If we don't want to create a new file at all, there is no need to
      go any further - bail out right here.  */
@@ -129,7 +151,7 @@ maybe_create_keybox (char *filename, int force, int *r_created)
                                            not happen though.  */
   save_slash = *last_slash_in_filename;
   *last_slash_in_filename = 0;
-  if (gnupg_access(filename, F_OK))
+  if (access(filename, F_OK))
     {
       static int tried;
 
@@ -138,9 +160,9 @@ maybe_create_keybox (char *filename, int force, int *r_created)
           tried = 1;
           try_make_homedir (filename);
         }
-      if ((ec = gnupg_access (filename, F_OK)))
+      if (access (filename, F_OK))
         {
-          rc = gpg_error (ec);
+          rc = gpg_error_from_syserror ();
           *last_slash_in_filename = save_slash;
           goto leave;
         }
@@ -183,7 +205,7 @@ maybe_create_keybox (char *filename, int force, int *r_created)
 
   /* The file does not yet exist, create it now. */
   oldmask = umask (077);
-  fp = es_fopen (filename, "wb");
+  fp = fopen (filename, "wb");
   if (!fp)
     {
       rc = gpg_error_from_syserror ();
@@ -200,7 +222,7 @@ maybe_create_keybox (char *filename, int force, int *r_created)
   rc = _keybox_write_header_blob (fp, 0);
   if (rc)
     {
-      es_fclose (fp);
+      fclose (fp);
       log_error (_("error creating keybox '%s': %s\n"),
                  filename, gpg_strerror (rc));
       goto leave;
@@ -211,7 +233,7 @@ maybe_create_keybox (char *filename, int force, int *r_created)
   if (r_created)
     *r_created = 1;
 
-  es_fclose (fp);
+  fclose (fp);
   rc = 0;
 
  leave:
@@ -278,15 +300,14 @@ keydb_add_resource (ctrl_t ctrl, const char *url, int force, int *auto_created)
   /* see whether we can determine the filetype */
   if (rt == KEYDB_RESOURCE_TYPE_NONE)
     {
-      estream_t fp;
+      FILE *fp = fopen( filename, "rb" );
 
-      fp = es_fopen( filename, "rb" );
       if (fp)
         {
           u32 magic;
 
           /* FIXME: check for the keybox magic */
-          if (es_fread (&magic, 4, 1, fp) == 1 )
+          if (fread (&magic, 4, 1, fp) == 1 )
             {
               if (magic == 0x13579ace || magic == 0xce9a5713)
                 ; /* GDBM magic - no more support */
@@ -295,8 +316,7 @@ keydb_add_resource (ctrl_t ctrl, const char *url, int force, int *auto_created)
             }
           else /* maybe empty: assume keybox */
             rt = KEYDB_RESOURCE_TYPE_KEYBOX;
-
-          es_fclose (fp);
+          fclose (fp);
         }
       else /* no file yet: create keybox */
         rt = KEYDB_RESOURCE_TYPE_KEYBOX;
